@@ -691,15 +691,6 @@ static void canonselphy_cleanup_job(const void *vjob) {
 	free((void*)job);
 }
 
-static void canonselphy_teardown(void *vctx) {
-	struct canonselphy_ctx *ctx = vctx;
-
-	if (!ctx)
-		return;
-
-	free(ctx);
-}
-
 static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, int copies)
 {
 	struct canonselphy_ctx *ctx = vctx;
@@ -725,8 +716,10 @@ static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, in
 	   job contents.  Ignore it if it comes through here.. */
 	i = read(data_fd, rdbuf, 4);
 	if (i != 4) {
-		if (i == 0)
+		if (i == 0) {
+			canonselphy_cleanup_job(job);
 			return CUPS_BACKEND_CANCEL;
+		}
 		ERROR("Read failed (%d/%d)\n", i, 4);
 		perror("ERROR: Read failed");
 		canonselphy_cleanup_job(job);
@@ -743,8 +736,10 @@ static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, in
 	/* Read the rest of the header.. */
 	i = read(data_fd, rdbuf + offset, MAX_HEADER - offset);
 	if (i != MAX_HEADER - offset) {
-		if (i == 0)
+		if (i == 0) {
+			canonselphy_cleanup_job(job);
 			return CUPS_BACKEND_CANCEL;
+		}
 		ERROR("Read failed (%d/%d)\n",
 		      i, MAX_HEADER - offset);
 		perror("ERROR: Read failed");
@@ -763,8 +758,8 @@ static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, in
 	}
 
 	if (printer_type != ctx->type) {
-		ERROR("Printer/Job mismatch (%d/%d)\n", ctx->type, ctx->printer->type);
-		free(job);
+		ERROR("Printer/Job mismatch (%d/%d/%d)\n", ctx->type, ctx->printer->type, printer_type);
+		canonselphy_cleanup_job(job);
 		return CUPS_BACKEND_CANCEL;
 	}
 
@@ -1161,7 +1156,6 @@ struct dyesub_backend canonselphy_backend = {
 	.cmdline_arg = canonselphy_cmdline_arg,
 	.init = canonselphy_init,
 	.attach = canonselphy_attach,
-	.teardown = canonselphy_teardown,
 	.read_parse = canonselphy_read_parse,
 	.cleanup_job = canonselphy_cleanup_job,
 	.main_loop = canonselphy_main_loop,
