@@ -1,7 +1,7 @@
 /*
  *   CUPS Backend common code
  *
- *   Copyright (c) 2007-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   Copyright (c) 2007-2019 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -27,19 +27,19 @@
  */
 
 #include "backend_common.h"
+#include <errno.h>
 
-#define BACKEND_VERSION "0.88G"
+#define BACKEND_VERSION "0.94G"
 #ifndef URI_PREFIX
 #error "Must Define URI_PREFIX"
 #endif
 
-#define NUM_CLAIM_ATTEMPTS 10
-
 #define URB_XFER_SIZE  (64*1024)
 #define XFER_TIMEOUT    15000
 
 #define USB_SUBCLASS_PRINTER 0x1
 #define USB_INTERFACE_PROTOCOL_BIDIR 0x2
+#define USB_INTERFACE_PROTOCOL_IPP   0x4
 
 /* Global Variables */
 int dyesub_debug = 0;
@@ -48,16 +48,18 @@ int fast_return = 0;
 int extra_vid = -1;
 int extra_pid = -1;
 int extra_type = -1;
-int copies = 1;
+int ncopies = 1;
+int collate = 0;
 int test_mode = 0;
 int old_uri = 0;
+int quiet = 0;
 
 static int max_xfer_size = URB_XFER_SIZE;
 static int xfer_timeout = XFER_TIMEOUT;
 
 /* Support Functions */
-static int backend_claim_interface(struct libusb_device_handle *dev, int iface,
-				   int num_claim_attempts)
+int backend_claim_interface(struct libusb_device_handle *dev, int iface,
+			    int num_claim_attempts)
 {
 	int ret;
 	do {
@@ -300,7 +302,7 @@ int send_data(struct libusb_device_handle *dev, uint8_t endp,
 		}
 
 		if (ret < 0) {
-			ERROR("Failure to send data to printer (libusb error %d: (%d/%d to 0x%02x))\n", ret, num, len, endp);
+			ERROR("Failure to send data to printer (libusb error %d: (%d/%d to 0x%02x))\n", ret, num, len2, endp);
 			return ret;
 		}
 		len -= num;
@@ -435,8 +437,16 @@ static int probe_device(struct libusb_device *device,
 				continue;
 			}
 
-#if 0
-			// Make sure it's a printer class device that supports bidir comms  (XXX Is this always true?)
+#if 1
+			/* Explicitly exclude IPP-over-USB interfaces */
+			if (desc->bDeviceClass == LIBUSB_CLASS_PER_INTERFACE &&
+			    config->interface[iface].altsetting[altset].bInterfaceClass == LIBUSB_CLASS_PRINTER &&
+			    config->interface[iface].altsetting[altset].bInterfaceSubClass == USB_SUBCLASS_PRINTER &&
+			    config->interface[iface].altsetting[altset].bInterfaceProtocol == USB_INTERFACE_PROTOCOL_IPP) {
+				continue;
+			}
+#else
+			// Make sure it's a printer class device that supports bidir comms  (XXX Is this necessarily true?)
 			if (desc->bDeviceClass == LIBUSB_CLASS_PRINTER ||
 			    (desc->bDeviceClass == LIBUSB_CLASS_PER_INTERFACE &&
 			     config->interface[iface].altsetting[altset].bInterfaceClass == LIBUSB_CLASS_PRINTER &&
@@ -544,9 +554,9 @@ candidate:
 	if (!descr || !strlen(descr)) { /* Last-ditch, generate */
 		char *product2 = url_decode(product);
 		char *manuf3 = url_decode(manuf);
-		descr = malloc(256);
+		descr = malloc(514); /* 256 + 256 + 1 + 1 */
 		if (!descr) {
-			ERROR("Memory allocation failure (%d bytes)\n", 256);
+			ERROR("Memory allocation failure (%d bytes)\n", 514);
 			if (manuf3)
 				free(manuf3);
 			if (product2)
@@ -554,7 +564,7 @@ candidate:
 			return -1;
 		}
 
-		sprintf(descr, "%s %s", manuf3, product2);
+		snprintf(descr, 514, "%s %s", manuf3, product2);
 		free(product2);
 		free(manuf3);
 	}
@@ -656,7 +666,16 @@ abort:
 	return found;
 }
 
-extern struct dyesub_backend updr150_backend;
+void generic_teardown(void *vctx)
+{
+	if (!vctx)
+		return;
+
+	free(vctx);
+}
+
+extern struct dyesub_backend sonyupd_backend;
+extern struct dyesub_backend sonyupdneo_backend;
 extern struct dyesub_backend kodak6800_backend;
 extern struct dyesub_backend kodak605_backend;
 extern struct dyesub_backend kodak1400_backend;
@@ -683,7 +702,7 @@ static struct dyesub_backend *backends[] = {
 	&shinkos2145_backend,
 	&shinkos6145_backend,
 	&shinkos6245_backend,
-	&updr150_backend,
+	&sonyupd_backend,
 	&mitsu70x_backend,
 	&mitsud90_backend,
 	&mitsu9550_backend,
@@ -812,7 +831,7 @@ static int query_markers(struct dyesub_backend *backend, void *ctx, int full)
 void print_license_blurb(void)
 {
 	const char *license = "\n\
-Copyright 2007-2018 Solomon Peachy <pizza AT shaftnet DOT org>\n\
+Copyright 2007-2019 Solomon Peachy <pizza AT shaftnet DOT org>\n\
 \n\
 This program is free software; you can redistribute it and/or modify it\n\
 under the terms of the GNU General Public License as published by the Free\n\
@@ -848,13 +867,13 @@ void print_help(char *argv0, struct dyesub_backend *backend)
 	if (!backend) {
 		int i;
 		DEBUG("Environment variables:\n");
-		DEBUG(" DYESUB_DEBUG EXTRA_PID EXTRA_VID EXTRA_TYPE BACKEND SERIAL OLD_URI_SCHEME\n");
+		DEBUG(" DYESUB_DEBUG EXTRA_PID EXTRA_VID EXTRA_TYPE BACKEND SERIAL OLD_URI_SCHEME BACKEND_QUIET\n");
 		DEBUG("CUPS Usage:\n");
 		DEBUG("\tDEVICE_URI=someuri %s job user title num-copies options [ filename ]\n", URI_PREFIX);
 		DEBUG("\n");
 		DEBUG("Standalone Usage:\n");
 		DEBUG("\t%s\n", URI_PREFIX);
-		DEBUG("  [ -D ] [ -G ] [ -f ]\n");
+		DEBUG("  [ -D ] [ -G ] [ -f ] [ -v ]\n");
 		DEBUG("  [ backend_specific_args ] \n");
 		DEBUG("  [ -d copies ] \n");
 		DEBUG("  [ - | infile ] \n");
@@ -964,14 +983,9 @@ int main (int argc, char **argv)
 	char *use_serno = NULL;
 	int  printer_type;
 
-	DEBUG("Multi-Call Dye-sublimation CUPS Backend version %s\n",
-	      BACKEND_VERSION);
-	DEBUG("Copyright 2007-2018 Solomon Peachy\n");
-	DEBUG("This free software comes with ABSOLUTELY NO WARRANTY! \n");
-	DEBUG("Licensed under the GNU GPL.  Run with '-G' for more details.\n");
-	DEBUG("\n");
-
-	/* First pass at cmdline parsing */
+	/* Handle environment variables  */
+	if (getenv("BACKEND_QUIET"))
+		quiet = atoi(getenv("BACKEND_QUIET"));
 	if (getenv("DYESUB_DEBUG"))
 		dyesub_debug = atoi(getenv("DYESUB_DEBUG"));
 	if (getenv("EXTRA_PID"))
@@ -998,6 +1012,13 @@ int main (int argc, char **argv)
 		exit(1);
 	}
 
+	DEBUG("Multi-Call Dye-sublimation CUPS Backend version %s\n",
+	      BACKEND_VERSION);
+	DEBUG("Copyright 2007-2019 Solomon Peachy\n");
+	DEBUG("This free software comes with ABSOLUTELY NO WARRANTY! \n");
+	DEBUG("Licensed under the GNU GPL.  Run with '-G' for more details.\n");
+	DEBUG("\n");
+
 	use_serno = getenv("SERIAL");
 	uri = getenv("DEVICE_URI");  /* CUPS backend mode! */
 	type = getenv("FINAL_CONTENT_TYPE"); /* CUPS content type -- ie raster or command */
@@ -1012,7 +1033,7 @@ int main (int argc, char **argv)
 		if (argv[base])
 			jobid = atoi(argv[base]);
 		if (argv[base + 3])
-			copies = atoi(argv[base + 3]);
+			ncopies = atoi(argv[base + 3]);
 		if (argc > 6)
 			fname = argv[base + 5];
 		else
@@ -1216,7 +1237,13 @@ bypass:
 
 	if (!fname) {
 		if (uri)
-			fprintf(stderr, "ERROR: No input file specified\n");
+			ERROR("ERROR: No input file specified\n");
+		goto done_claimed;
+	}
+
+	if (ncopies < 1) {
+		ERROR("ERROR: need to have at least 1 copy!\n");
+		ret = CUPS_BACKEND_FAILED;
 		goto done_claimed;
 	}
 
@@ -1226,7 +1253,7 @@ bypass:
 		if (data_fd < 0) {
 			perror("ERROR:Can't open input file");
 			ret = CUPS_BACKEND_FAILED;
-			goto done;
+			goto done_claimed;
 		}
 	}
 
@@ -1242,7 +1269,7 @@ bypass:
 	if (i < 0) {
 		perror("ERROR:Can't open input");
 		ret = CUPS_BACKEND_FAILED;
-		goto done;
+		goto done_claimed;
 	}
 
 	/* Ignore SIGPIPE */
@@ -1250,7 +1277,7 @@ bypass:
 	signal(SIGTERM, sigterm_handler);
 
 	/* Time for the main processing loop */
-	INFO("Printing started (%d copies)\n", copies);
+	INFO("Printing started (%d copies)\n", ncopies);
 
 	/* See if it's a CUPS command stream, and if yes, handle it! */
 	if (type && !strcmp("application/vnd.cups-command", type))
@@ -1262,7 +1289,7 @@ bypass:
 newpage:
 
 	/* Read in data */
-	if ((ret = backend->read_parse(backend_ctx, &job, data_fd, copies))) {
+	if ((ret = backend->read_parse(backend_ctx, &job, data_fd, ncopies))) {
 		if (current_page)
 			goto done_multiple;
 		else
@@ -1276,11 +1303,11 @@ newpage:
 
 	/* Create our own joblist if necessary */
 	if (!(backend->flags & BACKEND_FLAG_JOBLIST)) {
-		struct dyesub_joblist *list = dyesub_joblist_create(backend, backend_ctx);
+		struct dyesub_joblist *jlist = dyesub_joblist_create(backend, backend_ctx);
 		if (!list)
 			goto done_claimed;
-		dyesub_joblist_addjob(list, job);
-		job = list;
+		dyesub_joblist_addjob(jlist, job);
+		job = jlist;
 	}
 
 	/* Dump the full marker dump */
@@ -1303,7 +1330,7 @@ newpage:
 
 	/* Log the completed page */
 	if (!uri)
-		PAGE("%d %d\n", current_page, copies);
+		PAGE("%d %d\n", current_page, ncopies);
 
 	/* Dump a marker status update */
 	ret = query_markers(backend, backend_ctx, !current_page);
@@ -1320,7 +1347,7 @@ done_multiple:
 
 	/* Done printing, log the total number of pages */
 	if (!uri)
-		PAGE("total %d\n", current_page * copies);
+		PAGE("total %d\n", current_page * ncopies);
 	ret = CUPS_BACKEND_OK;
 
 done_claimed:
@@ -1333,7 +1360,10 @@ done_close:
 done:
 
 	if (backend && backend_ctx) {
-		backend->teardown(backend_ctx);
+		if (backend->teardown)
+			backend->teardown(backend_ctx);
+		else
+			generic_teardown(backend_ctx);
 //		STATE("-org.gutenprint-attached-to-device");
 	}
 
@@ -1354,7 +1384,7 @@ void dump_markers(struct marker *markers, int marker_count, int full)
 
 	ATTR("marker-colors=");
 	for (i = 0 ; i < marker_count; i++) {
-		DEBUG2(markers[i].color);
+		DEBUG2("%s", markers[i].color);
 		if ((i+1) < marker_count)
 			DEBUG2(",");
 	}
@@ -1433,6 +1463,35 @@ minimal:
 	}
 }
 
+int dyesub_read_file(char *filename, void *databuf, int datalen,
+		     int *actual_len)
+{
+	int len;
+	int fd = open(filename, O_RDONLY);
+
+	if (fd < 0) {
+		ERROR("Unable to open '%s'\n", filename);
+		return CUPS_BACKEND_FAILED;
+	}
+	len = read(fd, databuf, datalen);
+	if (len < 0) {
+		ERROR("Bad Read! (%d/%d)\n", len, errno);
+		close(fd);
+		return CUPS_BACKEND_FAILED;
+	}
+	if (!actual_len && (datalen != len)) {
+		ERROR("Read mismatch (%d vs %d)\n", len, datalen);
+		close(fd);
+		return CUPS_BACKEND_FAILED;
+	}
+	close(fd);
+
+	if (actual_len)
+		*actual_len = len;
+
+	return CUPS_BACKEND_OK;
+}
+
 uint16_t uint16_to_packed_bcd(uint16_t val)
 {
         uint16_t bcd;
@@ -1481,7 +1540,11 @@ struct dyesub_joblist *dyesub_joblist_create(struct dyesub_backend *backend, voi
 	list->backend = backend;
 	list->ctx = ctx;
 	list->num_entries = 0;
-	list->copies = 1;
+
+	if (collate)
+		list->copies = ncopies;
+	else
+		list->copies = 1;
 
 	return list;
 }
