@@ -1,7 +1,7 @@
 /*
  *   CUPS Backend common code
  *
- *   (c) 2013-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -43,21 +43,40 @@
 #define __BACKEND_COMMON_H
 
 #define STR_LEN_MAX 64
-#define STATE( ... ) fprintf(stderr, "STATE: " __VA_ARGS__ )
-#define ATTR( ... ) fprintf(stderr, "ATTR: " __VA_ARGS__ )
-#define PAGE( ... ) fprintf(stderr, "PAGE: " __VA_ARGS__ )
-#define DEBUG( ... ) fprintf(stderr, "DEBUG: " __VA_ARGS__ )
-#define DEBUG2( ... ) fprintf(stderr, __VA_ARGS__ )
-#define INFO( ... )  fprintf(stderr, "INFO: " __VA_ARGS__ )
-#define WARNING( ... )  fprintf(stderr, "WARNING: " __VA_ARGS__ )
+#define STATE( ... ) do { if (!quiet) fprintf(stderr, "STATE: " __VA_ARGS__ ); } while(0)
+#define ATTR( ... ) do { if (!quiet) fprintf(stderr, "ATTR: " __VA_ARGS__ ); } while(0)
+#define PAGE( ... ) do { if (!quiet) fprintf(stderr, "PAGE: " __VA_ARGS__ ); } while(0)
+#define DEBUG( ... ) do { if (!quiet) fprintf(stderr, "DEBUG: " __VA_ARGS__ ); } while(0)
+#define DEBUG2( ... ) do { if (!quiet) fprintf(stderr, __VA_ARGS__ ); } while(0)
+#define INFO( ... )  do { if (!quiet) fprintf(stderr, "INFO: " __VA_ARGS__ ); } while(0)
+#define WARNING( ... )  do { fprintf(stderr, "WARNING: " __VA_ARGS__ ); } while(0)
 #define ERROR( ... ) do { fprintf(stderr, "ERROR: " __VA_ARGS__ ); sleep(1); } while (0)
 
 #if (__BYTE_ORDER == __LITTLE_ENDIAN)
+#define le64_to_cpu(__x) __x
 #define le32_to_cpu(__x) __x
 #define le16_to_cpu(__x) __x
 #define be16_to_cpu(__x) ntohs(__x)
 #define be32_to_cpu(__x) ntohl(__x)
+#define be64_to_cpu(__x) ((__uint64_t)(                         \
+        (((__uint64_t)(__x) & (__uint64_t)0x00000000000000ffULL) << 56) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x000000000000ff00ULL) << 40) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x0000000000ff0000ULL) << 24) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x00000000ff000000ULL) <<  8) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x000000ff00000000ULL) >>  8) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x0000ff0000000000ULL) >> 24) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x00ff000000000000ULL) >> 40) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0xff00000000000000ULL) >> 56)))
 #else
+#define le64_to_cpu(__x) ((__uint64_t)(                         \
+        (((__uint64_t)(__x) & (__uint64_t)0x00000000000000ffULL) << 56) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x000000000000ff00ULL) << 40) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x0000000000ff0000ULL) << 24) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x00000000ff000000ULL) <<  8) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x000000ff00000000ULL) >>  8) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x0000ff0000000000ULL) >> 24) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0x00ff000000000000ULL) >> 40) |   \
+        (((__uint64_t)(__x) & (__uint64_t)0xff00000000000000ULL) >> 56)))
 #define le32_to_cpu(x)							\
 	({								\
 		uint32_t __x = (x);					\
@@ -74,6 +93,7 @@
 			(((uint16_t)(__x) & (uint16_t)0x00ff) <<  8) | \
 			(((uint16_t)(__x) & (uint16_t)0xff00) >>  8))); \
 	})
+#define be64_to_cpu(__x) __x
 #define be32_to_cpu(__x) __x
 #define be16_to_cpu(__x) __x
 #endif
@@ -132,7 +152,16 @@ enum {
 	P_DNP_DSRX1 = 41,
 	P_FUJI_ASK300 = 42,
 	P_MAGICARD = 43,
-	P_SONY_UPD89x = 44,
+	P_SONY_UPD895 = 44,
+	P_SONY_UPD897 = 45,
+	P_SONY_UPD898 = 46,
+	P_SONY_UPCR20L = 47,
+	P_SONY_UPDR80 = 48,
+	P_KODAK_8810 = 49,
+	P_KODAK_7000 = 50,
+	P_KODAK_701X = 51,
+	P_KODAK_6900 = 52,
+	P_SHINKO_S2245 = 53,
 	P_END,
 };
 
@@ -195,9 +224,19 @@ void dump_markers(struct marker *markers, int marker_count, int full);
 void print_license_blurb(void);
 void print_help(char *argv0, struct dyesub_backend *backend);
 
+int dyesub_read_file(char *filename, void *databuf, int datalen,
+		     int *actual_len);
+
 uint16_t uint16_to_packed_bcd(uint16_t val);
 uint32_t packed_bcd_to_uint32(char *in, int len);
 
+void generic_teardown(void *vctx);
+
+/* USB enumeration and attachment */
+#define NUM_CLAIM_ATTEMPTS 10
+int backend_claim_interface(struct libusb_device_handle *dev, int iface,
+			    int num_claim_attempts);
+
 /* Job list manipulation */
 struct dyesub_joblist *dyesub_joblist_create(struct dyesub_backend *backend, void *ctx);
 int dyesub_joblist_addjob(struct dyesub_joblist *list, const void *job);
@@ -211,8 +250,10 @@ extern int fast_return;
 extern int extra_vid;
 extern int extra_pid;
 extern int extra_type;
-extern int copies;
+extern int ncopies;
+extern int collate;
 extern int test_mode;
+extern int quiet;
 
 enum {
 	TEST_MODE_NONE = 0,
@@ -236,10 +277,10 @@ extern struct dyesub_backend BACKEND;
 #define CUPS_BACKEND_RETRY_CURRENT 7 /* Retry immediately */
 
 /* Argument processing */
-#define GETOPT_LIST_GLOBAL "d:DfGh"
+#define GETOPT_LIST_GLOBAL "d:DfGhv"
 #define GETOPT_PROCESS_GLOBAL \
 			case 'd': \
-				copies = atoi(optarg); \
+				ncopies = atoi(optarg); \
 				break; \
 			case 'D': \
 				dyesub_debug++; \
@@ -252,6 +293,9 @@ extern struct dyesub_backend BACKEND;
 				exit(0); \
 			case 'h': \
 				print_help(argv[0], &BACKEND); \
-				exit(0);
+				exit(0); \
+			case 'v': \
+				quiet++; \
+				break;
 
 #endif /* __BACKEND_COMMON_H */
