@@ -1,7 +1,7 @@
 /*
  *   Citizen / DNP Photo Printer CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
  *
  *   Development of this backend was sponsored by:
  *
@@ -68,6 +68,7 @@ struct dnpds40_printjob {
 	int printspeed;
 	int can_rewind;
 	int buf_needed;
+	int cut_paper;
 };
 
 struct dnpds40_ctx {
@@ -86,6 +87,7 @@ struct dnpds40_ctx {
 	/* State */
 	uint32_t media;
 	uint32_t duplex_media;
+	int      duplex_media_status;
 	uint16_t media_count_new;
 
 	uint32_t last_multicut;
@@ -95,7 +97,8 @@ struct dnpds40_ctx {
 	int correct_count;
 	int needs_mlot;
 
-	struct marker marker;
+	struct marker marker[2];
+	int marker_count;
 
 	/* Printer capabilities */
 	uint32_t native_width;
@@ -175,7 +178,6 @@ struct dnpds40_cmd {
 #define MULTICUT_A4       41
 #define MULTICUT_A4x5X2   43
 
-
 #define MULTICUT_S_SIMPLEX  100
 #define MULTICUT_S_FRONT    200
 #define MULTICUT_S_BACK     300
@@ -195,31 +197,14 @@ struct dnpds40_cmd {
 
 #define min(__x, __y) ((__x) < (__y)) ? __x : __y
 
-/* Legacy CW-01 spool file support */
-struct cw01_spool_hdr {
-	uint8_t  type; /* 0x00 -> 0x06 */
-	uint8_t  res; /* vertical resolution; 0x00 == 334dpi, 0x01 == 600dpi */
-	uint8_t  copies; /* number of prints */
-	uint8_t  null0;
-	uint32_t plane_len; /* LE */
-	uint8_t  null1[4];
-};
+/* Legacy spool file support */
+static int legacy_cw01_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data);
+static int legacy_dnp_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data);
+static int legacy_dnp620_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data);
+static int legacy_dnp820_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data);
 
-#define DPI_334 0
-#define DPI_600 1
-
-#define TYPE_DSC  0
-#define TYPE_L    1
-#define TYPE_PC   2
-#define TYPE_2DSC 3
-#define TYPE_3L   4
-#define TYPE_A5   5
-#define TYPE_A6   6
-/* Legacy CW-01 spool file support */
-
-static int cw01_read_parse(struct dnpds40_printjob *job, int data_fd,
-			   struct cw01_spool_hdr *hdr, int read_data);
 static void dnpds40_cleanup_job(const void *vjob);
+static int dnpds40_query_markers(void *vctx, struct marker **markers, int *count);
 
 #define JOB_EQUIV(__x)  if (job1->__x != job2->__x) goto done
 
@@ -419,7 +404,7 @@ static void dnpds40_cleanup_string(char *start, int len)
 	}
 }
 
-static char *dnpds40_printer_type(int type)
+static const char *dnpds40_printer_type(int type)
 {
 	switch(type) {
 	case P_DNP_DS40: return "DS40";
@@ -435,7 +420,7 @@ static char *dnpds40_printer_type(int type)
 	return "Unknown";
 }
 
-static char *dnpds40_media_types(int media)
+static const char *dnpds40_media_types(int media)
 {
 	switch (media) {
 	case 100: return "UNKNOWN100"; // seen in driver dumps
@@ -455,7 +440,7 @@ static char *dnpds40_media_types(int media)
 	return "Unknown type";
 }
 
-static char *dnpds620_media_extension_code(int media)
+static const char *dnpds620_media_extension_code(int media)
 {
 	switch (media) {
 	case 00: return "Normal Paper";
@@ -468,7 +453,7 @@ static char *dnpds620_media_extension_code(int media)
 	return "Unknown type";
 }
 
-static char *dnpds820_media_subtypes(int media)
+static const char *dnpds820_media_subtypes(int media)
 {
 	switch (media) {
 	case 0001: return "SD";
@@ -480,7 +465,7 @@ static char *dnpds820_media_subtypes(int media)
 	return "Unknown type";
 }
 
-static char *dnpds80_duplex_media_types(int media)
+static const char *dnpds80_duplex_media_types(int media)
 {
 	switch (media) {
 	case 100: return "8x10.75";
@@ -492,7 +477,22 @@ static char *dnpds80_duplex_media_types(int media)
 	return "Unknown type";
 }
 
-static char *dnpds80_duplex_statuses(int status)
+#define DUPLEX_UNIT_PAPER_NONE 0
+#define DUPLEX_UNIT_PAPER_PROTECTIVE 1
+#define DUPLEX_UNIT_PAPER_PRESENT 2
+
+static const char *dnpds80_duplex_paper_status(int media)
+{
+	switch (media) {
+	case DUPLEX_UNIT_PAPER_NONE: return "No Paper";
+	case DUPLEX_UNIT_PAPER_PROTECTIVE: return "Protective Sheet";
+	case DUPLEX_UNIT_PAPER_PRESENT: return "Cut Paper Present";
+	default:
+		return "Unknown";
+	}
+}
+
+static const char *dnpds80_duplex_statuses(int status)
 {
 	switch (status) {
 	case 5000: return "No Error";
@@ -564,7 +564,7 @@ static char *dnpds80_duplex_statuses(int status)
 	return "Unknown Duplexer Error";
 }
 
-static char *dnpds40_statuses(int status)
+static const char *dnpds40_statuses(int status)
 {
 	if (status >= 5000 && status <= 5999)
 		return dnpds80_duplex_statuses(status);
@@ -746,6 +746,41 @@ static int dnpds40_query_mqty(struct dnpds40_ctx *ctx)
 	return count;
 }
 
+static int dnpds80dx_query_paper(struct dnpds40_ctx *ctx)
+{
+	struct dnpds40_cmd cmd;
+	uint8_t *resp;
+	int len = 0;
+
+	/* Query Duplex Media Info */
+	dnpds40_build_cmd(&cmd, "INFO", "UNIT_CUT_PAPER", 0);
+
+	resp = dnpds40_resp_cmd(ctx, &cmd, &len);
+	if (resp) {
+		char tmp[5];
+		char status;
+
+		dnpds40_cleanup_string((char*)resp, len);
+
+		memcpy(tmp, resp + 4, 4);
+		status = tmp[3];
+		tmp[3] = '0';
+		tmp[4] = 0;
+
+		ctx->duplex_media = atoi(tmp);
+
+		tmp[0] = tmp[1] = tmp[2] = '0';
+		tmp[3] = status;
+		ctx->duplex_media_status = atoi(tmp);
+
+		free(resp);
+	} else {
+		return CUPS_BACKEND_FAILED;
+	}
+
+	return CUPS_BACKEND_OK;
+}
+
 static int dnpds40_attach(void *vctx, struct libusb_device_handle *dev, int type,
 			  uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
 {
@@ -819,32 +854,8 @@ static int dnpds40_attach(void *vctx, struct libusb_device_handle *dev, int type
 		}
 
 		if (ctx->type == P_DNP_DS80D) {
-			struct dnpds40_cmd cmd;
-			uint8_t *resp;
-			int len = 0;
-
-			/* Query Duplex Media Info */
-			dnpds40_build_cmd(&cmd, "INFO", "CUT_PAPER", 0);
-
-			resp = dnpds40_resp_cmd(ctx, &cmd, &len);
-			if (resp) {
-				char tmp[5];
-
-				dnpds40_cleanup_string((char*)resp, len);
-
-				memcpy(tmp, resp + 4, 4);
-				tmp[4] = 0;
-
-				ctx->duplex_media = atoi(tmp);
-
-				/* Subtract out the paper status */
-				if (ctx->duplex_media & 3)
-					ctx->duplex_media -= (ctx->duplex_media & 3);
-
-				free(resp);
-			} else {
+			if (dnpds80dx_query_paper(ctx))
 				return CUPS_BACKEND_FAILED;
-			}
 		}
 
 #if (defined(DNP_ONLY) || defined(CITIZEN_ONLY))
@@ -948,7 +959,7 @@ static int dnpds40_attach(void *vctx, struct libusb_device_handle *dev, int type
 		break;
 	case P_DNP_DS620:
 		ctx->native_width = 1920;
-		ctx->max_height = 5480;
+		ctx->max_height = 5604;
 		ctx->correct_count = 1;
 		ctx->supports_counterp = 1;
 		ctx->supports_matte = 1;
@@ -1153,10 +1164,19 @@ static int dnpds40_attach(void *vctx, struct libusb_device_handle *dev, int type
 	}
 
 	/* Fill out marker structure */
-	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
-	ctx->marker.name = dnpds40_media_types(ctx->media);
-	ctx->marker.levelmax = ctx->media_count_new;
-	ctx->marker.levelnow = -2;
+	ctx->marker[0].color = "#00FFFF#FF00FF#FFFF00";
+	ctx->marker[0].name = dnpds40_media_types(ctx->media);
+	ctx->marker[0].levelmax = ctx->media_count_new;
+	ctx->marker[0].levelnow = -2;
+	ctx->marker_count = 1;
+
+	if (ctx->type == P_DNP_DS80D) {
+		ctx->marker[1].color = "#00FFFF#FF00FF#FFFF00";
+		ctx->marker[1].name = dnpds80_duplex_media_types(ctx->duplex_media);
+		ctx->marker[1].levelmax = ctx->marker[0].levelmax/2;
+		ctx->marker[1].levelnow = -2;
+		ctx->marker_count++;
+	}
 
 	return CUPS_BACKEND_OK;
 }
@@ -1230,6 +1250,7 @@ static int dnpds40_read_parse(void *vctx, const void **vjob, int data_fd, int co
 
 	job->databuf = malloc(MAX_PRINTJOB_LEN);
 	if (!job->databuf) {
+		dnpds40_cleanup_job(job);
 		ERROR("Memory allocation failure!\n");
 		return CUPS_BACKEND_RETRY_CURRENT;
 	}
@@ -1250,29 +1271,36 @@ static int dnpds40_read_parse(void *vctx, const void **vjob, int data_fd, int co
 			return CUPS_BACKEND_CANCEL;
 		}
 
-		if (job->databuf[job->datalen + 0] != 0x1b ||
-		    job->databuf[job->datalen + 1] != 0x50) {
-			struct cw01_spool_hdr hdr;
-			/* See if it's the "classic" CW01 header */
-			memcpy(&hdr, job->databuf + job->datalen, sizeof(hdr));
-			hdr.plane_len = le32_to_cpu(hdr.plane_len);
-
-			if (hdr.type > 0x06 ||
-			    hdr.res > 0x01 ||
-			    hdr.null1[0] || hdr.null1[1] || hdr.null1[2] || hdr.null1[3]) {
-				ERROR("Unrecognized header data format @%d!\n", job->datalen);
-				dnpds40_cleanup_job(job);
-			} else {
-				job->dpi = (hdr.res == DPI_600) ? 600 : 334;
-				i = cw01_read_parse(job, data_fd, &hdr, i);
-				if (i == CUPS_BACKEND_OK)
-					goto parsed;
-				else {
-					dnpds40_cleanup_job(job);
-					return i;
+		/* Special case handling for beginning of job */
+		if (job->datalen == 0) {
+			/* See if job lacks the standard ESC-P start sequence */
+			if (job->databuf[job->datalen + 0] != 0x1b ||
+			    job->databuf[job->datalen + 1] != 0x50) {
+				switch(ctx->type) {
+				case P_CITIZEN_CW01:
+					i = legacy_cw01_read_parse(job, data_fd, i);
+					break;
+				case P_DNP_DS620:
+					i = legacy_dnp620_read_parse(job, data_fd, i);
+					break;
+				case P_DNP_DS820:
+					i = legacy_dnp820_read_parse(job, data_fd, i);
+					break;
+				case P_DNP_DSRX1:
+				case P_DNP_DS40:
+				case P_DNP_DS80:
+				case P_DNP_DS80D:
+				default:
+					i = legacy_dnp_read_parse(job, data_fd, i);
+					break;
 				}
+
+				if (i == CUPS_BACKEND_OK) {
+					goto parsed;
+				}
+				dnpds40_cleanup_job(job);
+				return i;
 			}
-			return CUPS_BACKEND_CANCEL;
 		}
 
 		/* Parse out length of data chunk, if any */
@@ -1285,7 +1313,7 @@ static int dnpds40_read_parse(void *vctx, const void **vjob, int data_fd, int co
 			i = read(data_fd, job->databuf + job->datalen + sizeof(struct dnpds40_cmd),
 				 remain);
 			if (i < 0) {
-				ERROR("Data Read Error: %d (%d/%d @%d)\n", i, remain, j, job->datalen);
+				ERROR("Data Read Error: %d (%d/%d @%d/%d)\n", i, remain, j, job->datalen,MAX_PRINTJOB_LEN);
 				dnpds40_cleanup_job(job);
 				return i;
 			}
@@ -1372,7 +1400,11 @@ static int dnpds40_read_parse(void *vctx, const void **vjob, int data_fd, int co
 			case 11808:
 				job->dpi = 300;
 				break;
+			case 13146:
+				job->dpi = 334;
+				break;
 			case 23615:
+			case 23616:
 				job->dpi = 600;
 				break;
 			default:
@@ -1439,7 +1471,7 @@ parsed:
 	/* And sanity-check whatever value is there */
 	if (job->printspeed == 0 && job->dpi == 600) {
 		job->printspeed = 1;
-	} else if (job->printspeed == 1 && job->dpi == 300) {
+	} else if (job->printspeed >= 1 && job->dpi == 300) {
 		job->printspeed = 0;
 	}
 
@@ -1455,11 +1487,14 @@ parsed:
 	}
 
 	/* Only DS80D supports Cut Paper types */
-	if (job->multicut > 100 &&
-	    ctx->type != P_DNP_DS80D) {
-		ERROR("Only DS80D supports cut-paper sizes!\n");
-		dnpds40_cleanup_job(job);
-		return CUPS_BACKEND_CANCEL;
+	if (job->multicut > 100) {
+		if ( ctx->type == P_DNP_DS80D) {
+			job->cut_paper = 1;
+		} else {
+			ERROR("Only DS80D supports cut-paper sizes!\n");
+			dnpds40_cleanup_job(job);
+			return CUPS_BACKEND_CANCEL;
+		}
 	}
 
 	/* Figure out the number of buffers we need. */
@@ -1887,12 +1922,16 @@ top:
 
 	{
 		/* Figure out remaining native prints */
-		ctx->marker.levelnow = dnpds40_query_mqty(ctx);
-		if (ctx->marker.levelnow < 0)
+		if (dnpds40_query_markers(ctx, NULL, NULL))
 			return CUPS_BACKEND_FAILED;
-		dump_markers(&ctx->marker, 1, 0);
+		if (ctx->marker[0].levelnow < 0)
+			return CUPS_BACKEND_FAILED;
+		dump_markers(ctx->marker, ctx->marker_count, 0);
 
-		count = ctx->marker.levelnow; // For logic below.
+		// For logic below.
+		count = ctx->marker[0].levelnow;
+		if (job->cut_paper && count > ctx->marker[1].levelnow)
+			count = ctx->marker[1].levelnow;
 
 		/* See if we can rewind to save media */
 		if (job->can_rewind && ctx->supports_rewind) {
@@ -1944,6 +1983,13 @@ top:
 		}
 	}
 
+	/* Work around a bug in older gutenprint releases. */
+	if (ctx->last_multicut >= 200 && ctx->last_multicut < 300 &&
+	    multicut >= 200 && multicut < 300) {
+		WARNING("Bogus multicut value for duplex page, correcting\n");
+		multicut += 100;
+	}
+
 	/* Store our last multicut state */
 	ctx->last_multicut = multicut;
 
@@ -1995,7 +2041,7 @@ top:
 
 	/* Program in the multicut setting, if one exists */
 	if (multicut) {
-		snprintf(buf, sizeof(buf), "%08u", multicut);
+		snprintf(buf, sizeof(buf), "%08d", multicut);
 		dnpds40_build_cmd(&cmd, "IMAGE", "MULTICUT", 8);
 		if ((ret = dnpds40_do_cmd(ctx, &cmd, (uint8_t*)buf, 8)))
 			return CUPS_BACKEND_FAILED;
@@ -2046,26 +2092,9 @@ top:
 		}
 
 		/* Figure out remaining native prints */
-		dnpds40_build_cmd(&cmd, "INFO", "MQTY", 0);
-
-		resp = dnpds40_resp_cmd(ctx, &cmd, &len);
-		if (!resp)
+		if (dnpds40_query_markers(ctx, NULL, NULL))
 			return CUPS_BACKEND_FAILED;
-
-		dnpds40_cleanup_string((char*)resp, len);
-
-		count = atoi((char*)resp+4);
-		free(resp);
-
-		if (count) {
-			/* Old-sk00l models report one less than they should */
-			if (!ctx->correct_count)
-				count++;
-
-			count -= ctx->mediaoffset;
-		}
-		ctx->marker.levelnow = count;
-		dump_markers(&ctx->marker, 1, 0);
+		dump_markers(ctx->marker, ctx->marker_count, 0);
 	}
 
 	/* Clean up */
@@ -2528,6 +2557,7 @@ static int dnpds40_get_status(struct dnpds40_ctx *ctx)
 		count = atoi((char*)resp);
 
 		INFO("Duplexer Status: %s\n", dnpds80_duplex_statuses(count));
+		INFO("Duplexer Media Status: %s\n", dnpds80_duplex_paper_status(ctx->duplex_media_status));
 
 		free(resp);
 	}
@@ -2589,8 +2619,10 @@ static int dnpds40_get_status(struct dnpds40_ctx *ctx)
 	}
 
 	/* Report Cut Media */
-	if (ctx->type == P_DNP_DS80D)
-		INFO("Duplex Media Type: %s\n", dnpds80_duplex_media_types(ctx->media));
+	if (ctx->type == P_DNP_DS80D) {
+		INFO("Duplex Media Type: %s\n", dnpds80_duplex_media_types(ctx->duplex_media));
+		INFO("Duplexer Media Status: %s\n", dnpds80_duplex_paper_status(ctx->duplex_media_status));
+	}
 
 	if (ctx->media_count_new)
 		INFO("Native Prints Available on New Media: %u\n", ctx->media_count_new);
@@ -2888,18 +2920,18 @@ static int dnpds40_cmdline_arg(void *vctx, int argc, char **argv)
 			j = dnpds40_get_sensors(ctx);
 			break;
 		case 'k': {
-			int time = atoi(optarg);
+			int sleeptime = atoi(optarg);
 			if (!ctx->supports_standby) {
 				ERROR("Printer does not support standby\n");
 				j = -1;
 				break;
 			}
-			if (time < 0 || time > 99) {
+			if (sleeptime < 0 || sleeptime > 99) {
 				ERROR("Value out of range (0-99)");
 				j = -1;
 				break;
 			}
-			j = dnpds620_standby_mode(ctx, time);
+			j = dnpds620_standby_mode(ctx, sleeptime);
 			break;
 		}
 		case 'K': {
@@ -2979,13 +3011,32 @@ static int dnpds40_query_markers(void *vctx, struct marker **markers, int *count
 {
 	struct dnpds40_ctx *ctx = vctx;
 
-	*markers = &ctx->marker;
-	*count = 1;
+	if (markers)
+		*markers = ctx->marker;
+	if (count)
+		*count = ctx->marker_count;
 
-	ctx->marker.levelnow = dnpds40_query_mqty(ctx);
-	if (ctx->marker.levelnow < 0)
+	ctx->marker[0].levelnow = dnpds40_query_mqty(ctx);
+
+	if (ctx->marker[0].levelnow < 0)
 		return CUPS_BACKEND_FAILED;
 
+        if (ctx->type == P_DNP_DS80D) {
+		if (dnpds80dx_query_paper(ctx))
+			return CUPS_BACKEND_FAILED;
+		switch (ctx->duplex_media_status) {
+		case DUPLEX_UNIT_PAPER_NONE:
+			ctx->marker[1].levelnow = 0;
+			break;
+		case DUPLEX_UNIT_PAPER_PROTECTIVE:
+			ctx->marker[1].levelnow = -1;
+			break;
+		case DUPLEX_UNIT_PAPER_PRESENT:
+			ctx->marker[1].levelnow = -3;
+			break;
+		}
+	}
+
 	return CUPS_BACKEND_OK;
 }
 
@@ -3019,7 +3070,7 @@ static const char *dnpds40_prefixes[] = {
 /* Exported */
 struct dyesub_backend dnpds40_backend = {
 	.name = "DNP DS-series / Citizen C-series",
-	.version = "0.109",
+	.version = "0.117",
 	.uri_prefixes = dnpds40_prefixes,
 	.flags = BACKEND_FLAG_JOBLIST,
 	.cmdline_usage = dnpds40_cmdline,
@@ -3046,25 +3097,29 @@ struct dyesub_backend dnpds40_backend = {
 	}
 };
 
-/* Legacy CW-01 spool file support */
-
-static int cw01_read_parse(struct dnpds40_printjob *job, int data_fd,
-			   struct cw01_spool_hdr *hdr, int read_data)
+/* Windows spool file support */
+static int legacy_spool_helper(struct dnpds40_printjob *job, int data_fd,
+			       int read_data, int hdrlen, uint32_t plane_len,
+			       int parse_dpi)
 {
+	uint8_t *buf;
+	uint8_t bmp_hdr[14];
 	int i, remain;
 	uint32_t j;
-	uint8_t *buf;
-	uint8_t plane_hdr[14];
 
-	remain = hdr->plane_len * 3;
+	/* Allocate a temp processing buffer */
+	remain = plane_len * 3;
 	buf = malloc(remain);
 	if (!buf) {
 		ERROR("Memory allocation failure!\n");
 		return CUPS_BACKEND_RETRY_CURRENT;
 	}
-	j = read_data - sizeof(*hdr);
-	memcpy(buf, job->databuf, j);
+
+	/* Copy over the post-jobhdr crap into our processing buffer */
+	j = read_data - hdrlen;
+	memcpy(buf, job->databuf + hdrlen, j);
 	remain -= j;
+
 	/* Read in the remaining spool data */
 	while (remain) {
 		i = read(data_fd, buf + j, remain);
@@ -3078,58 +3133,67 @@ static int cw01_read_parse(struct dnpds40_printjob *job, int data_fd,
 		j += i;
 	}
 
-	/* Generate plane header (same for all planes) */
-	j = cpu_to_le32(hdr->plane_len) + 24;
-	memset(plane_hdr, 0, sizeof(plane_hdr));
-	plane_hdr[0] = 0x42;
-	plane_hdr[1] = 0x4d;
-	memcpy(plane_hdr + 2, &j, sizeof(j));
-	plane_hdr[10] = 0x40;
-	plane_hdr[11] = 0x04;
+	if (parse_dpi) {
+		/* Parse out Y DPI */
+		memcpy(&j, buf + 28, sizeof(j));
+		j = le32_to_cpu(j);
+		switch(j) {
+		case 11808:
+			job->dpi = 300;
+			break;
+		case 23615:
+		case 23616:
+			job->dpi = 600;
+			break;
+		default:
+			ERROR("Unrecognized printjob resolution (%u ppm)\n", j);
+			free(buf);
+			return CUPS_BACKEND_CANCEL;
+		}
+	}
 
-	/* Okay, generate a new stream into job->databuf! */
-#if 0
-	job->datalen += sprintf((char*)job->databuf + job->datalen,
-				"\033PCNTRL QTY             00000008%07d\r", hdr->copies);
-	job->datalen += sprintf((char*)job->databuf + job->datalen,
-				"\033PCNTRL CUTTER          0000000800000000");
-#else
-	/* QTY is stripped from the stream, and CUTTER is stashed away */
-	job->cutter = 0;
-#endif
+	/* Generate bitmap file header (same for all planes) */
+	j = cpu_to_le32(plane_len + 24);
+	memset(bmp_hdr, 0, sizeof(bmp_hdr));
+	bmp_hdr[0] = 0x42;
+	bmp_hdr[1] = 0x4d;
+	memcpy(bmp_hdr + 2, &j, sizeof(j));
+	bmp_hdr[10] = 0x40;
+	bmp_hdr[11] = 0x04;
 
+	/* Set up planes */
 	j = 0;
 
 	/* Y plane */
 	job->datalen += sprintf((char*)job->databuf + job->datalen,
-				"\033PIMAGE YPLANE          %08u", hdr->plane_len + 24);
-	memcpy(job->databuf + job->datalen, plane_hdr, sizeof(plane_hdr));
-	job->datalen += sizeof(plane_hdr);
-	memcpy(job->databuf + job->datalen, buf + j, hdr->plane_len);
-	job->datalen += hdr->plane_len;
-	j += hdr->plane_len;
+				"\033PIMAGE YPLANE          %08u", plane_len + 24);
+	memcpy(job->databuf + job->datalen, bmp_hdr, sizeof(bmp_hdr));
+	job->datalen += sizeof(bmp_hdr);
+	memcpy(job->databuf + job->datalen, buf + j, plane_len);
+	job->datalen += plane_len;
+	j += plane_len;
 	memset(job->databuf + job->datalen, 0, 10);
 	job->datalen += 10;
 
 	/* M plane */
 	job->datalen += sprintf((char*)job->databuf + job->datalen,
-				"\033PIMAGE MPLANE          %08u", hdr->plane_len + 24);
-	memcpy(job->databuf + job->datalen, plane_hdr, sizeof(plane_hdr));
-	job->datalen += sizeof(plane_hdr);
-	memcpy(job->databuf + job->datalen, buf + j, hdr->plane_len);
-	job->datalen += hdr->plane_len;
-	j += hdr->plane_len;
+				"\033PIMAGE MPLANE          %08u", plane_len + 24);
+	memcpy(job->databuf + job->datalen, bmp_hdr, sizeof(bmp_hdr));
+	job->datalen += sizeof(bmp_hdr);
+	memcpy(job->databuf + job->datalen, buf + j, plane_len);
+	job->datalen += plane_len;
+	j += plane_len;
 	memset(job->databuf + job->datalen, 0, 10);
 	job->datalen += 10;
 
 	/* C plane */
 	job->datalen += sprintf((char*)job->databuf + job->datalen,
-				"\033PIMAGE CPLANE          %08u", hdr->plane_len + 24);
-	memcpy(job->databuf + job->datalen, plane_hdr, sizeof(plane_hdr));
-	job->datalen += sizeof(plane_hdr);
-	memcpy(job->databuf + job->datalen, buf + j, hdr->plane_len);
-	job->datalen += hdr->plane_len;
-	j += hdr->plane_len;
+				"\033PIMAGE CPLANE          %08u", plane_len + 24);
+	memcpy(job->databuf + job->datalen, bmp_hdr, sizeof(bmp_hdr));
+	job->datalen += sizeof(bmp_hdr);
+	memcpy(job->databuf + job->datalen, buf + j, plane_len);
+	job->datalen += plane_len;
+	j += plane_len;
 	memset(job->databuf + job->datalen, 0, 10);
 	job->datalen += 10;
 
@@ -3143,6 +3207,183 @@ static int cw01_read_parse(struct dnpds40_printjob *job, int data_fd,
 	return CUPS_BACKEND_OK;
 }
 
+struct cw01_spool_hdr {
+	uint8_t  type; /* TYPE_??? */
+	uint8_t  res; /* DPI_??? */
+	uint8_t  copies; /* number of prints */
+	uint8_t  null0;
+	uint32_t plane_len; /* LE */
+	uint8_t  null1[4];
+};
+
+#define DPI_334 0
+#define DPI_600 1
+
+#define TYPE_DSC  0
+#define TYPE_L    1
+#define TYPE_PC   2
+#define TYPE_2DSC 3
+#define TYPE_3L   4
+#define TYPE_A5   5
+#define TYPE_A6   6
+
+static int legacy_cw01_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data)
+{
+	struct cw01_spool_hdr hdr;
+	uint32_t plane_len;
+
+	/* get original header out of structure */
+	memcpy(&hdr, job->databuf + job->datalen, sizeof(hdr));
+
+	/* Early parsing and sanity checking */
+	plane_len = le32_to_cpu(hdr.plane_len);
+
+	if (hdr.type > TYPE_A6 ||
+	    hdr.res > DPI_600 ||
+	    hdr.null1[0] || hdr.null1[1] || hdr.null1[2] || hdr.null1[3]) {
+		ERROR("Unrecognized header data format @%d!\n", job->datalen);
+		return CUPS_BACKEND_CANCEL;
+	}
+	job->dpi = (hdr.res == DPI_600) ? 600 : 334;
+	job->cutter = 0;
+
+	return legacy_spool_helper(job, data_fd, read_data,
+				   sizeof(hdr), plane_len, 0);
+}
+
+struct rx1_spool_hdr {
+	uint8_t  type; /* equals MULTICUT_?? - 1 */
+	uint8_t  null0;
+	uint8_t  copies; /* number of copies, always fixed at 01.. */
+	uint8_t  null1;
+	uint32_t plane_len; /* LE */
+        uint8_t  flags; /* combination of FLAG_?? */
+	uint8_t  null2[3];
+};
+
+#define FLAG_MATTE     0x02
+#define FLAG_NORETRY   0x08
+#define FLAG_2INCH     0x10
+
+static int legacy_dnp_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data)
+{
+	struct rx1_spool_hdr hdr;
+	uint32_t plane_len;
+
+	/* get original header out of structure */
+	memcpy(&hdr, job->databuf + job->datalen, sizeof(hdr));
+
+	/* Early parsing and sanity checking */
+	plane_len = le32_to_cpu(hdr.plane_len);
+
+	if (hdr.type >= MULTICUT_A4x5X2 ||
+	    hdr.null2[0] || hdr.null2[1] || hdr.null2[2]) {
+		ERROR("Unrecognized header data format @%d!\n", job->datalen);
+		return CUPS_BACKEND_CANCEL;
+	}
+
+	/* Don't bother with FW version checks for legacy stuff */
+	job->multicut = hdr.type + 1;
+	job->matte = (hdr.flags & FLAG_MATTE) ? 1 : 0;
+	job->cutter = (hdr.flags & FLAG_2INCH) ? 120 : 0;
+
+	return legacy_spool_helper(job, data_fd, read_data,
+				   sizeof(hdr), plane_len, 1);
+}
+
+struct ds620_spool_hdr {
+	uint8_t  type; /* MULTICUT_?? -1, but >0x90 is a flag for rewind */
+	uint8_t  copies; /* Always fixed at 01..*/
+	uint8_t  null0[2];
+	uint8_t  quality;  /* 0x02 is HQ, 0x00 is HS.  Equivalent to DPI. */
+	uint8_t  unk[3];   /* Always 00 01 00 */
+	uint32_t plane_len; /* LE */
+	uint8_t  flags; /* FLAG_?? */
+	uint8_t  null1[3];
+};
+#define FLAG_LUSTER    0x04
+#define FLAG_FINEMATTE 0x06
+
+static int legacy_dnp620_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data)
+{
+	struct ds620_spool_hdr hdr;
+	uint32_t plane_len;
+
+	/* get original header out of structure */
+	memcpy(&hdr, job->databuf + job->datalen, sizeof(hdr));
+
+	/* Early parsing and sanity checking */
+	plane_len = le32_to_cpu(hdr.plane_len);
+
+	/* Used to signify rewind request.  Backend handles automatically. */
+	if (hdr.type > 0x90)
+		hdr.type -= 0x90;
+
+	if (hdr.type > MULTICUT_A4x5X2 ||
+	    hdr.null1[0] || hdr.null1[1] || hdr.null1[2]) {
+		ERROR("Unrecognized header data format @%d!\n", job->datalen);
+		return CUPS_BACKEND_CANCEL;
+	}
+
+	/* Don't bother with FW version checks for legacy stuff */
+	job->multicut = hdr.type + 1;
+	if ((hdr.flags & FLAG_FINEMATTE) == FLAG_FINEMATTE)
+		job->matte = 21;
+	else if (hdr.flags & FLAG_LUSTER)
+		job->matte = 22;
+	else if (hdr.flags & FLAG_MATTE)
+		job->matte = 1;
+
+	job->cutter = (hdr.flags & FLAG_2INCH) ? 120 : 0;
+
+	return legacy_spool_helper(job, data_fd, read_data,
+				   sizeof(hdr), plane_len, 1);
+}
+
+#define FLAG_820_HD     0x80
+#define FLAG_820_RETRY  0x20
+#define FLAG_820_LUSTER 0x06
+#define FLAG_820_FMATTE 0x04
+#define FLAG_820_MATTE  0x02
+
+static int legacy_dnp820_read_parse(struct dnpds40_printjob *job, int data_fd, int read_data)
+{
+	struct ds620_spool_hdr hdr;
+	uint32_t plane_len;
+
+	/* get original header out of structure */
+	memcpy(&hdr, job->databuf + job->datalen, sizeof(hdr));
+
+	/* Early parsing and sanity checking */
+	plane_len = le32_to_cpu(hdr.plane_len);
+
+	/* Used to signify rewind request.  Backend handles automatically. */
+	if (hdr.type > 0x90)
+		hdr.type -= 0x90;
+
+	if (hdr.type > MULTICUT_A4x5X2 ||
+	    hdr.null1[0] || hdr.null1[1] || hdr.null1[2]) {
+		ERROR("Unrecognized header data format @%d!\n", job->datalen);
+		return CUPS_BACKEND_CANCEL;
+	}
+
+	/* Don't bother with FW version checks for legacy stuff */
+	job->multicut = hdr.type + 1;
+	if ((hdr.flags & FLAG_820_FMATTE) == FLAG_820_FMATTE)
+		job->matte = 21;
+	else if (hdr.flags & FLAG_820_LUSTER)
+		job->matte = 22;
+	else if (hdr.flags & FLAG_820_MATTE)
+		job->matte = 1;
+
+	if (hdr.flags & FLAG_820_HD)
+		job->printspeed = 3;
+
+	return legacy_spool_helper(job, data_fd, read_data,
+				   sizeof(hdr), plane_len, 1);
+}
+
+
 /*
 
 Basic spool file format for CW01
@@ -3151,11 +3392,11 @@ TT RR NN 00 XX XX XX XX  00 00 00 00              <- FILE header.
 
   NN          : copies (0x01 or more)
   RR          : resolution; 0 == 334 dpi, 1 == 600dpi
-  TT          : type 0x02 == 4x6, 0x01 == 5x3.5
+  TT          : type 0x02 == 4x6, 0x01 == 5x3.5, 0x06 = 6x9
   XX XX XX XX : plane length (LE)
                 plane length * 3 + 12 == file length.
 
-Followed by three planes, each with this header:
+Followed by three planes, each with this 40b header:
 
 28 00 00 00 00 08 00 00  RR RR 00 00 01 00 08 00
 00 00 00 00 00 00 00 00  5a 33 00 00 YY YY 00 00
