@@ -1,7 +1,7 @@
 /*
  *   Mitsubishi CP-9xxx Photo Printer Family CUPS backend
  *
- *   (c) 2014-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2014-2019 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -43,21 +43,59 @@
 
 #define BACKEND mitsu9550_backend
 
+
+#if defined(USE_DLOPEN)
+#define WITH_DYNAMIC
+#include <dlfcn.h>
+#define DL_INIT() do {} while(0)
+#define DL_OPEN(__x) dlopen(__x, RTLD_NOW)
+#define DL_SYM(__x, __y) dlsym(__x, __y)
+#define DL_CLOSE(__x) dlclose(__x)
+#define DL_EXIT() do {} while(0)
+#elif defined(USE_LTDL)
+#define WITH_DYNAMIC
+#include <ltdl.h>
+#define DL_INIT() lt_dlinit()
+#define DL_OPEN(__x) lt_dlopen(__x)
+#define DL_SYM(__x, __y) lt_dlsym(__x, __y)
+#define DL_CLOSE(__x) do {} while(0)
+#define DL_EXIT() lt_dlexit()
+#else
+#define DL_INIT()     do {} while(0)
+#define DL_CLOSE(__x) do {} while(0)
+#define DL_EXIT()     do {} while(0)
+#warning "No dynamic loading support!"
+#endif
+
 #include "backend_common.h"
 
-#define USB_VID_MITSU        0x06D3
-#define USB_PID_MITSU_9500D  0x0393
-#define USB_PID_MITSU_9000D  0x0394
-#define USB_PID_MITSU_9000AM 0x0395
-#define USB_PID_MITSU_9550D  0x03A1
-#define USB_PID_MITSU_9550DS 0x03A5  // or DZ/DZS/DZU
-#define USB_PID_MITSU_9600D  0x03A9
-//#define USB_PID_MITSU_9600DS  XXXXXX
-#define USB_PID_MITSU_9800D  0x03AD
-#define USB_PID_MITSU_9800DS 0x03AE
-#define USB_PID_MITSU_98__D  0x3B21
-//#define USB_PID_MITSU_9810D   XXXXXX
-//#define USB_PID_MITSU_9820DS  XXXXXX
+// #include "lib70x/libMitsuD70ImageReProcess.h"
+
+#ifndef LUT_LEN
+#define COLORCONV_RGB 0
+#define COLORCONV_BGR 1
+
+#define LUT_LEN 14739
+struct BandImage {
+	   void  *imgbuf;
+	 int32_t bytes_per_row;
+	uint16_t origin_cols;
+	uint16_t origin_rows;
+	uint16_t cols;
+	uint16_t rows;
+};
+#endif
+
+#define REQUIRED_LIB_APIVERSION 4
+
+/* Image processing library function prototypes */
+#define LIB_NAME_RE "libMitsuD70ImageReProcess.so" // Reimplemented library
+
+typedef int (*lib70x_getapiversionFN)(void);
+typedef int (*Get3DColorTableFN)(uint8_t *buf, const char *filename);
+typedef struct CColorConv3D *(*Load3DColorTableFN)(const uint8_t *ptr);
+typedef void (*Destroy3DColorTableFN)(struct CColorConv3D *this);
+typedef void (*DoColorConvFN)(struct CColorConv3D *this, uint8_t *data, uint16_t cols, uint16_t rows, uint32_t bytes_per_row, int rgb_bgr);
 
 #ifndef CORRTABLE_PATH
 #ifdef PACKAGE_DATA_DIR
@@ -73,80 +111,90 @@
 #define LAMINATE_STRIDE 1868
 #define DATATABLE_SIZE  42204
 
+/* USB VIDs and PIDs */
+
+#define USB_VID_MITSU        0x06D3
+#define USB_PID_MITSU_9500D  0x0393
+#define USB_PID_MITSU_9000D  0x0394
+#define USB_PID_MITSU_9000AM 0x0395
+#define USB_PID_MITSU_9550D  0x03A1
+#define USB_PID_MITSU_9550DS 0x03A5  // or DZ/DZS/DZU
+#define USB_PID_MITSU_9600D  0x03A9
+//#define USB_PID_MITSU_9600DS  XXXXXX
+#define USB_PID_MITSU_9800D  0x03AD
+#define USB_PID_MITSU_9800DS 0x03AE
+#define USB_PID_MITSU_98__D  0x3B21
+//#define USB_PID_MITSU_9810D   XXXXXX
+//#define USB_PID_MITSU_9820DS  XXXXXX
+
 /* Spool file structures */
 
 /* Print parameters1 */
 struct mitsu9550_hdr1 {
-	uint8_t  cmd[4]; /* 1b 57 20 2e */
+	uint8_t  cmd[4];  /* 1b 57 20 2e */
 	uint8_t  unk[10]; /* 00 0a 10 00 [...] */
-	uint16_t cols; /* BE */
-	uint16_t rows; /* BE */
-	uint8_t  matte;  /* CP9810/9820 only. 01 for matte, 00 glossy */
+	uint16_t cols;    /* BE */
+	uint16_t rows;    /* BE */
+	uint8_t  matte;   /* CP9810/9820 only. 01 for matte, 00 glossy */
 	uint8_t  null[31];
 } __attribute__((packed));
 
 /* Print parameters2 */
 struct mitsu9550_hdr2 {
-	uint8_t  cmd[4]; /* 1b 57 21 2e */
-	uint8_t  unk[24]; /* 00 80 00 22 08 03 [...] */
-	uint16_t copies; /* BE, 1-680 */
+	uint8_t  cmd[4];   /* 1b 57 21 2e */
+	uint8_t  unk[24];  /* 00 80 00 22 08 03 [...] */
+	uint16_t copies;   /* BE, 1-680 */
 	uint8_t  null[2];
-	uint8_t  cut; /* 00 == normal, 83 == 2x6*2 */
+	uint8_t  cut;      /* 00 == normal, 83 == 2x6*2 */
 	uint8_t  unkb[5];
-	uint8_t  mode; /* 00 == fine, 80 == superfine */
+	uint8_t  mode;     /* 00 == fine, 80 == superfine */
 	uint8_t  unkc[11]; /* 00 [...] 00 01 */
 } __attribute__((packed));
 
 /* Fine Deep selection (9550 only) */
 struct mitsu9550_hdr3 {
-	uint8_t  cmd[4]; /* 1b 57 22 2e */
-	uint8_t  unk[7]; /* 00 40 00 [...] */
-	uint8_t  mode2;  /* 00 == normal, 01 == finedeep */
+	uint8_t  cmd[4];   /* 1b 57 22 2e */
+	uint8_t  unk[7];   /* 00 40 00 [...] */
+	uint8_t  mode2;    /* 00 == normal, 01 == finedeep */
 	uint8_t  null[38];
 } __attribute__((packed));
 
 /* Error policy? */
 struct mitsu9550_hdr4 {
-	uint8_t  cmd[4]; /* 1b 57 26 2e */
-	uint8_t  unk[46]; /* 00 70 00 00 00 00 00 00 01 01 00 [...] */
+	uint8_t  cmd[4];   /* 1b 57 26 2e */
+	uint8_t  unk[46];  /* 00 70 00 00 00 00 00 00 01 01 00 [...] */
 } __attribute__((packed));
 
 /* Data plane header */
 struct mitsu9550_plane {
-	uint8_t  cmd[4]; /* 1b 5a 54 XX */  /* XX == 0x10 if 16bpp, 0x00 for 8bpp */
+	uint8_t  cmd[4];     /* 1b 5a 54 XX */  /* XX == 0x10 if 16bpp, 0x00 for 8bpp */
 	uint16_t col_offset; /* BE, normally 0, where we start dumping data */
 	uint16_t row_offset; /* BE, normally 0, where we start dumping data */
 	uint16_t cols;       /* BE */
 	uint16_t rows;       /* BE */
 } __attribute__((packed));
 
-/* CP98xx Tabular Data */
+/* CP98xx Tabular Data, as stored in data file! */
 struct mitsu98xx_data {
-	uint16_t GNMby[256];   // @0
-	uint16_t GNMgm[256];   // @512
-	uint16_t GNMrc[256];   // @1024
-	double   GammaParams[3]; // @1536
-	uint8_t  KH[2048];     // @1560
-	uint32_t unk_b[3];     // @3608
-
-	struct {
-		double  unka[256];  // @0
-		double  unkb[256];  // @2048
-		uint32_t unkc[10];  // @4096
-		double  unkd[256];  // @4136
-		double  unke[256];  // @6184  // *= sharp->coef[X]
-		uint32_t unkf[10];  // @8232
-		double  unkg[256];  // @8272
-		                    // @10320
-	} WMAM; // @3620
-	uint8_t  unc_d[4];    // @13940  @10320 (from wmam start)
-	struct {
-		uint32_t unk_a;      // @13944/10324 (padding?)
-		double   coef[10];   // @13948/10328 (sharpness coefficients, level 0-9)
-		uint32_t unk_b[5];   // @14028/10408
-	} sharp; // total 104, @13944/10324
-	uint8_t  unk_e[20];   // @14048/10428
-	                      // @14068/10448
+	/* @    0 */	uint16_t GNMby[256];     /* BGR Order uncertain */
+	/* @  512 */	uint16_t GNMgm[256];
+	/* @ 1024 */    uint16_t GNMrc[256];
+	/* @ 1536 */    uint16_t unk_sharp[20];  /* Actual format is: u16, u16[9], u16, u16[9] */
+	/* @ 1576 */    double   GammaAdj[3];    /* Assumed to be same order as tables (BGR?) */
+	/* @ 1600 */	struct {
+		/* @    0 */	double   unka[256];
+		/* @ 2048 */	double   unkb[256];
+		/* @ 4096 */	uint32_t unkc[10];
+		/* @ 4136 */	double   unkd[256];
+		/* @ 6184 */	double   unke[256]; // *= sharp->coef[X]
+		/* @ 8232 */	uint32_t unkf[10];
+		/* @ 8272 */	double   unkg[256];
+		/* @10320 */
+			} WMAM;
+	/* @11920 */	double   sharp_coef[11]; /* 0 is off, 1-10 are the levels.  Default is 5. [4 in settings] */
+	/* @12008 */	uint32_t unk_kh[3];
+	/* @12020 */	uint8_t  KH[2048];
+	/* @14068 */
 } __attribute__((packed));
 
 struct mitsu98xx_tables {
@@ -194,8 +242,15 @@ struct mitsu9550_ctx {
 	struct marker marker;
 
 	/* CP98xx stuff */
-	struct mitsu98xx_tables *m98xxdata;
+	void *dl_handle;
+	lib70x_getapiversionFN GetAPIVersion;
+	Get3DColorTableFN Get3DColorTable;
+	Load3DColorTableFN Load3DColorTable;
+	Destroy3DColorTableFN Destroy3DColorTable;
+	DoColorConvFN DoColorConv;
+
 	struct CColorConv3D *lut;
+	struct mitsu98xx_tables *m98xxdata;
 };
 
 /* Printer data structures */
@@ -290,11 +345,9 @@ static void mitsu98xx_dogamma(uint8_t *src, uint16_t *dest, uint8_t plane,
 {
 	src += plane;
 	while(len--) {
-		*dest++ = table[*src];
+		*dest++ = cpu_to_be16(table[*src]);
 		src += 3;
 	}
-	/* TODO:  Eventually, when we do real processing of this data, we will need to
-	   have the gamma table in native endian format and generate BE data at the end. */
 }
 
 static int mitsu98xx_fillmatte(struct mitsu9550_printjob *job)
@@ -302,7 +355,7 @@ static int mitsu98xx_fillmatte(struct mitsu9550_printjob *job)
 	int fd, i;
 	uint32_t j, remain;
 
-	DEBUG("Reading %d bytes of matte data from disk (%d/%d)\n", job->cols * job->rows, job->cols, LAMINATE_STRIDE);
+	DEBUG("Reading %d bytes of matte data from disk (%d/%d)\n", job->cols * job->rows * 2, job->cols, LAMINATE_STRIDE);
 	fd = open(MITSU_M98xx_LAMINATE_FILE, O_RDONLY);
 	if (fd < 0) {
 		WARNING("Unable to open matte lamination data file '%s'\n", MITSU_M98xx_LAMINATE_FILE);
@@ -318,8 +371,8 @@ static int mitsu98xx_fillmatte(struct mitsu9550_printjob *job)
 	matte->cmd[3] = 0x10;
 	matte->row_offset = 0;
 	matte->col_offset = 0;
-	matte->cols = job->hdr1.cols;
-	matte->rows = job->hdr1.rows;
+	matte->cols = cpu_to_be16(job->hdr1.cols);
+	matte->rows = cpu_to_be16(job->hdr1.rows);
 	job->datalen += sizeof(struct mitsu9550_plane);
 
 	/* Read in the matte data plane */
@@ -355,173 +408,12 @@ done:
 	return CUPS_BACKEND_OK;
 }
 
-/*** 3D color Lookup table stuff.  Taken out of lib70x ****/
+#ifndef LUT_LEN
 #define LUT_LEN 14739
 #define COLORCONV_RGB 0
 #define COLORCONV_BGR 1
-
-struct CColorConv3D {
-	uint8_t lut[17][17][17][3];
-};
-
-/* Load the Lookup table off of disk into *PRE-ALLOCATED* buffer */
-int CColorConv3D_Get3DColorTable(uint8_t *buf, const char *filename)
-{
-	FILE *stream;
-
-	if (!filename)
-		return 1;
-	if (!*filename)
-		return 2;
-	if (!buf)
-		return 3;
-
-	stream = fopen(filename, "rb");
-	if (!stream)
-		return 4;
-
-	fseek(stream, 0, SEEK_END);
-	if (ftell(stream) < LUT_LEN) {
-		fclose(stream);
-		return 5;
-	}
-	fseek(stream, 0, SEEK_SET);
-	fread(buf, 1, LUT_LEN, stream);
-	fclose(stream);
-
-	return 0;
-}
-
-/* Parse the on-disk LUT data into the structure.... */
-struct CColorConv3D *CColorConv3D_Load3DColorTable(const uint8_t *ptr)
-{
-	struct CColorConv3D *this;
-	this = malloc(sizeof(*this));
-	if (!this)
-		return NULL;
-
-	int i, j, k;
-
-	for (i = 0 ; i <= 16 ; i++) {
-		for (j = 0 ; j <= 16 ; j++) {
-			for (k = 0; k <= 16; k++) {
-				this->lut[k][j][i][2] = *ptr++;
-				this->lut[k][j][i][1] = *ptr++;
-				this->lut[k][j][i][0] = *ptr++;
-			}
-		}
-	}
-	return this;
-}
-void CColorConv3D_Destroy3DColorTable(struct CColorConv3D *this)
-{
-	free(this);
-}
-
-/* Transform a single pixel. */
-static void CColorConv3D_DoColorConvPixel(struct CColorConv3D *this, uint8_t *redp, uint8_t *grnp, uint8_t *blup)
-{
-	int red_h;
-	int grn_h;
-	int blu_h;
-	int grn_li;
-	int red_li;
-	int blu_li;
-	int red_l;
-	int grn_l;
-	int blu_l;
-
-	uint8_t *tab0;       // @ 14743
-	uint8_t *tab1;       // @ 14746
-	uint8_t *tab2;       // @ 14749
-	uint8_t *tab3;       // @ 14752
-	uint8_t *tab4;       // @ 14755
-	uint8_t *tab5;       // @ 14758
-	uint8_t *tab6;       // @ 14761
-	uint8_t *tab7;       // @ 14764
-
-	red_h = *redp >> 4;
-	red_l = *redp & 0xF;
-	red_li = 16 - red_l;
-
-	grn_h = *grnp >> 4;
-	grn_l = *grnp & 0xF;
-	grn_li = 16 - grn_l;
-
-	blu_h = *blup >> 4;
-	blu_l = *blup & 0xF;
-	blu_li = 16 - blu_l;
-
-//	printf("%d %d %d =>", *redp, *grnp, *blup);
-
-	tab0 = this->lut[red_h+0][grn_h+0][blu_h+0];
-	tab1 = this->lut[red_h+1][grn_h+0][blu_h+0];
-	tab2 = this->lut[red_h+0][grn_h+1][blu_h+0];
-	tab3 = this->lut[red_h+1][grn_h+1][blu_h+0];
-	tab4 = this->lut[red_h+0][grn_h+0][blu_h+1];
-	tab5 = this->lut[red_h+1][grn_h+0][blu_h+1];
-	tab6 = this->lut[red_h+0][grn_h+1][blu_h+1];
-	tab7 = this->lut[red_h+1][grn_h+1][blu_h+1];
-
-#if 0
-	printf(" %d %d %d ", tab0[0], tab0[1], tab0[2]);
-	printf(" %d %d %d ", tab1[0], tab1[1], tab1[2]);
-	printf(" %d %d %d ", tab2[0], tab2[1], tab2[2]);
-	printf(" %d %d %d ", tab3[0], tab3[1], tab3[2]);
-	printf(" %d %d %d ", tab4[0], tab4[1], tab4[2]);
-	printf(" %d %d %d ", tab5[0], tab5[1], tab5[2]);
-	printf(" %d %d %d ", tab6[0], tab6[1], tab6[2]);
-	printf(" %d %d %d ", tab7[0], tab7[1], tab7[2]);
 #endif
-	*redp = (blu_li
-		 * (grn_li * (red_li * tab0[0] + red_l * tab1[0])
-		    + grn_l * (red_li * tab2[0] + red_l * tab3[0]))
-		 + blu_l
-		 * (grn_li * (red_li * tab4[0] + red_l * tab5[0])
-		    + grn_l * (red_li * tab6[0] + red_l * tab7[0]))
-		 + 2048) >> 12;
-	*grnp = (blu_li
-		 * (grn_li * (red_li * tab0[1] + red_l * tab1[1])
-		    + grn_l * (red_li * tab2[1] + red_l * tab3[1]))
-		 + blu_l
-		 * (grn_li * (red_li * tab4[1] + red_l * tab5[1])
-		    + grn_l * (red_li * tab6[1] + red_l * tab7[1]))
-		 + 2048) >> 12;
-	*blup = (blu_li
-		 * (grn_li * (red_li * tab0[2] + red_l * tab1[2])
-		    + grn_l * (red_li * tab2[2] + red_l * tab3[2]))
-		 + blu_l
-		 * (grn_li * (red_li * tab4[2] + red_l * tab5[2])
-		    + grn_l * (red_li * tab6[2] + red_l * tab7[2]))
-		 + 2048) >> 12;
 
-//	printf("=> %d %d %d\n", *redp, *grnp, *blup);
-}
-
-/* Perform a total conversion on an entire image */
-void CColorConv3D_DoColorConv(struct CColorConv3D *this, uint8_t *data, uint16_t cols, uint16_t rows, uint32_t stride, int rgb_bgr)
-{
-	uint16_t i, j;
-
-	uint8_t *ptr;
-
-	for ( i = 0; i < rows ; i++ )
-	{
-		ptr = data;
-		for ( j = 0; cols > j; j++ )
-		{
-			if (rgb_bgr) {
-				CColorConv3D_DoColorConvPixel(this, ptr + 2, ptr + 1, ptr);
-			} else {
-				CColorConv3D_DoColorConvPixel(this, ptr, ptr + 1, ptr + 2);
-			}
-			ptr += 3;
-		}
-		data += stride;
-	}
-}
-
-/* ---- end 3D LUT ---- */
 static int mitsu9550_get_status(struct mitsu9550_ctx *ctx, uint8_t *resp, int status, int status2, int media);
 static char *mitsu9550_media_types(uint8_t type, uint8_t is_s);
 
@@ -534,6 +426,8 @@ static void *mitsu9550_init(void)
 	}
 	memset(ctx, 0, sizeof(struct mitsu9550_ctx));
 
+	DL_INIT();
+
 	return ctx;
 }
 
@@ -559,6 +453,46 @@ static int mitsu9550_attach(void *vctx, struct libusb_device_handle *dev, int ty
 	    ctx->type == P_MITSU_9810)
 		ctx->is_98xx = 1;
 
+	/* Attempt to open the library */
+#if defined(WITH_DYNAMIC)
+	if (!ctx->is_98xx) goto skip;
+
+	DEBUG("Attempting to load image processing library\n");
+	ctx->dl_handle = DL_OPEN(LIB_NAME_RE);
+	if (!ctx->dl_handle)
+		WARNING("Image processing library not found, using internal fallback code\n");
+	if (ctx->dl_handle) {
+		ctx->GetAPIVersion = DL_SYM(ctx->dl_handle, "lib70x_getapiversion");
+		if (!ctx->GetAPIVersion) {
+			ERROR("Problem resolving API Version symbol in imaging processing library, too old or not installed?\n");
+			DL_CLOSE(ctx->dl_handle);
+			ctx->dl_handle = NULL;
+			return CUPS_BACKEND_FAILED;
+		}
+		if (ctx->GetAPIVersion() != REQUIRED_LIB_APIVERSION) {
+			ERROR("Image processing library API version mismatch!\n");
+			DL_CLOSE(ctx->dl_handle);
+			ctx->dl_handle = NULL;
+			return CUPS_BACKEND_FAILED;
+		}
+
+		ctx->Get3DColorTable = DL_SYM(ctx->dl_handle, "CColorConv3D_Get3DColorTable");
+		ctx->Load3DColorTable = DL_SYM(ctx->dl_handle, "CColorConv3D_Load3DColorTable");
+		ctx->Destroy3DColorTable = DL_SYM(ctx->dl_handle, "CColorConv3D_Destroy3DColorTable");
+		ctx->DoColorConv = DL_SYM(ctx->dl_handle, "CColorConv3D_DoColorConv");
+		if (!ctx->Get3DColorTable || !ctx->Load3DColorTable ||
+		    !ctx->Destroy3DColorTable || !ctx->DoColorConv ) {
+			ERROR("Problem resolving symbols in imaging processing library\n");
+			DL_CLOSE(ctx->dl_handle);
+			ctx->dl_handle = NULL;
+			return CUPS_BACKEND_FAILED;
+		} else {
+			DEBUG("Image processing library successfully loaded\n");
+		}
+	}
+skip:
+#endif
+
 	if (test_mode < TEST_MODE_NOATTACH) {
 		if (mitsu9550_get_status(ctx, (uint8_t*) &media, 0, 0, 1))
 			return CUPS_BACKEND_FAILED;
@@ -596,10 +530,14 @@ static void mitsu9550_teardown(void *vctx) {
 	if (!ctx)
 		return;
 
-	if (ctx->lut)
-		CColorConv3D_Destroy3DColorTable(ctx->lut);
-	if (ctx->m98xxdata)
-		free(ctx->m98xxdata);
+	if (ctx->dl_handle) {
+		if (ctx->lut)
+			ctx->Destroy3DColorTable(ctx->lut);
+		if (ctx->m98xxdata)
+			free(ctx->m98xxdata);
+		DL_CLOSE(ctx->dl_handle);
+	}
+
 	free(ctx);
 }
 
@@ -700,31 +638,56 @@ hdr_done:
 
 	/* Read in CP98xx data tables if necessary */
 	if (ctx->is_98xx && !job->is_raw && !ctx->m98xxdata) {
-		int fd;
-
-		DEBUG("Reading in 98xx data from disk\n");
-		fd = open(MITSU_M98xx_DATATABLE_FILE, O_RDONLY);
-		if (fd < 0) {
-			ERROR("Unable to open 98xx data table file '%s'\n", MITSU_M98xx_DATATABLE_FILE);
-			mitsu9550_cleanup_job(job);
-			return CUPS_BACKEND_FAILED;
-		}
+		int ret;
 		ctx->m98xxdata = malloc(DATATABLE_SIZE);
 		if (!ctx->m98xxdata) {
 			ERROR("Memory allocation Failure!\n");
 			mitsu9550_cleanup_job(job);
 			return CUPS_BACKEND_RETRY_CURRENT;
 		}
-		remain = DATATABLE_SIZE;
-		while (remain) {
-			i = read(fd, ((uint8_t*)ctx->m98xxdata) + (DATATABLE_SIZE - remain), remain);
-			if (i < 0) {
-				mitsu9550_cleanup_job(job);
-				return CUPS_BACKEND_CANCEL;
-			}
-			remain -= i;
+
+		DEBUG("Reading in 98xx data from disk\n");
+		if ((ret = dyesub_read_file(MITSU_M98xx_DATATABLE_FILE, ctx->m98xxdata, DATATABLE_SIZE, NULL))) {
+			ERROR("Unable to read 98xx data table file '%s'\n", MITSU_M98xx_DATATABLE_FILE);
+			free(ctx->m98xxdata);
+			return ret;
 		}
-		close(fd);
+
+		/* Byteswap data table to native endianness, if necessary */
+#if (__BYTE_ORDER == __LITTLE_ENDIAN)
+		int j;
+		struct mitsu98xx_data *ptr = &ctx->m98xxdata->superfine;
+		for (j = 0 ; j < 3 ; j++) {
+			for (i = 3 ; i < 3 ; i++) {
+				ptr->GammaAdj[i] = be64_to_cpu(ptr->GammaAdj[i]);
+				ptr->unk_kh[i] = be32_to_cpu(ptr->unk_kh[i]);
+			}
+			for (i = 0 ; i < 10 ; i++) {
+				ptr->WMAM.unkc[i] = be32_to_cpu(ptr->WMAM.unkc[i]);
+				ptr->WMAM.unkf[i] = be32_to_cpu(ptr->WMAM.unkf[i]);
+			}
+			for (i = 0 ; i < 11 ; i++) {
+				ptr->sharp_coef[i] = be64_to_cpu(ptr->sharp_coef[i]);
+			}
+			for (i = 0 ; i < 20 ; i++) {
+				ptr->unk_sharp[i] = be16_to_cpu(ptr->unk_sharp[i]);
+			}
+			for (i = 0 ; i < 256 ; i++) {
+				ptr->WMAM.unka[i] = be64_to_cpu(ptr->WMAM.unka[i]);
+				ptr->WMAM.unkb[i] = be64_to_cpu(ptr->WMAM.unkb[i]);
+				ptr->WMAM.unkd[i] = be64_to_cpu(ptr->WMAM.unkd[i]);
+				ptr->WMAM.unke[i] = be64_to_cpu(ptr->WMAM.unke[i]);
+				ptr->WMAM.unkg[i] = be64_to_cpu(ptr->WMAM.unkg[i]);
+
+				ptr->GNMby[i] = be16_to_cpu(ptr->GNMby[i]);
+				ptr->GNMgm[i] = be16_to_cpu(ptr->GNMgm[i]);
+				ptr->GNMrc[i] = be16_to_cpu(ptr->GNMrc[i]);
+
+			}
+			// XXX TODO: KH[2048]
+			ptr++;
+		}
+#endif
 	}
 
 	if (job->is_raw) {
@@ -832,8 +795,10 @@ hdr_done:
 			memcpy(job->databuf + job->datalen, buf, 4);
 			job->datalen += 4;
 
-			/* Unless we have a matte plane following, we're done */
-			if (job->hdr1.matte != 0x01)
+			/* Unless we have a raw matte plane following,
+			   we're done */
+			if (job->hdr1.matte != 0x01 ||
+			    !job->is_raw)
 				break;
 			remain = sizeof(buf);
 		} else {
@@ -856,31 +821,40 @@ hdr_done:
 		}
 	}
 
-	/* Apply LUT */
+	/* Apply LUT, if job calls for it.. */
 	if (ctx->is_98xx && !job->is_raw && job->hdr2.unkc[9]) {
-		DEBUG("Applying 3D LUT\n");
+
+		if (!ctx->dl_handle) {
+			// XXXFALLBACK write fallback code?
+			ERROR("!!! Image Processing Library not found, aborting!\n");
+			mitsu9550_cleanup_job(job);
+			return CUPS_BACKEND_CANCEL;
+		}
+
 		if (!ctx->lut) {
-			uint8_t *buf = malloc(LUT_LEN);
-			if (!buf) {
+			uint8_t *lbuf = malloc(LUT_LEN);
+			if (!lbuf) {
 				ERROR("Memory allocation failure!\n");
 				mitsu9550_cleanup_job(job);
 				return CUPS_BACKEND_RETRY_CURRENT;
 			}
-			if (CColorConv3D_Get3DColorTable(buf, MITSU_M98xx_LUT_FILE)) {
+			if (ctx->Get3DColorTable(lbuf, MITSU_M98xx_LUT_FILE)) {
 				ERROR("Unable to open LUT file '%s'\n", MITSU_M98xx_LUT_FILE);
 				mitsu9550_cleanup_job(job);
 				return CUPS_BACKEND_CANCEL;
 			}
-			ctx->lut = CColorConv3D_Load3DColorTable(buf);
-			free(buf);
+			ctx->lut = ctx->Load3DColorTable(lbuf);
+			free(lbuf);
 			if (!ctx->lut) {
 				ERROR("Unable to parse LUT\n");
 				mitsu9550_cleanup_job(job);
 				return CUPS_BACKEND_CANCEL;
 			}
 		}
-		CColorConv3D_DoColorConv(ctx->lut, job->databuf + sizeof(struct mitsu9550_plane),
-					 job->cols, job->rows, job->cols * 3, COLORCONV_BGR);
+
+		DEBUG("Running print data through 3D LUT\n");
+		ctx->DoColorConv(ctx->lut, job->databuf + sizeof(struct mitsu9550_plane),
+				 job->cols, job->rows, job->cols * 3, COLORCONV_BGR);
 		job->hdr2.unkc[9] = 0;
 	}
 
@@ -1199,7 +1173,7 @@ static int mitsu9550_main_loop(void *vctx, const void *vjob) {
 	int i, remain, planelen;
 
 	planelen = job->rows * job->cols * 2;
-	remain = (job->hdr1.matte ? 3 : 4) * (planelen + sizeof(struct mitsu9550_plane)) + sizeof(struct mitsu9550_cmd);
+	remain = (job->hdr1.matte ? 4 : 3) * (planelen + sizeof(struct mitsu9550_plane)) + sizeof(struct mitsu9550_cmd) * (job->hdr1.matte? 2 : 1) + LAMINATE_STRIDE * 2;
 	newbuf = malloc(remain);
 	if (!newbuf) {
 		ERROR("Memory allocation Failure!\n");
@@ -1254,7 +1228,7 @@ static int mitsu9550_main_loop(void *vctx, const void *vjob) {
 	newlen += planelen;
 
 	/* And finally, the job footer. */
-	memcpy(newbuf + newlen, job->databuf + sizeof(struct mitsu9550_plane) + planelen * 3, sizeof(struct mitsu9550_cmd));
+	memcpy(newbuf + newlen, job->databuf + sizeof(struct mitsu9550_plane) + planelen/2 * 3, sizeof(struct mitsu9550_cmd));
 	newlen += sizeof(struct mitsu9550_cmd);
 
 	/* Clean up, and move pointer to new buffer; */
@@ -1362,7 +1336,6 @@ top:
 	/* Send over plane data */
 	while(ptr < (job->databuf + job->datalen)) {
 		struct mitsu9550_plane *plane = (struct mitsu9550_plane *)ptr;
-		uint32_t planelen;
 		if (plane->cmd[0] != 0x1b ||
 		    plane->cmd[1] != 0x5a ||
 		    plane->cmd[2] != 0x54)
@@ -1457,7 +1430,7 @@ top:
 	/* Don't forget the 9810's matte plane */
 	if (job->hdr1.matte) {
 		struct mitsu9550_plane *plane = (struct mitsu9550_plane *)ptr;
-		uint32_t planelen = be16_to_cpu(plane->rows) * be16_to_cpu(plane->cols);
+		planelen = be16_to_cpu(plane->rows) * be16_to_cpu(plane->cols);
 
 		if (plane->cmd[3] == 0x10)
 			planelen *= 2;
@@ -1735,7 +1708,7 @@ static const char *mitsu9550_prefixes[] = {
 /* Exported */
 struct dyesub_backend mitsu9550_backend = {
 	.name = "Mitsubishi CP9xxx family",
-	.version = "0.41",
+	.version = "0.47",
 	.uri_prefixes = mitsu9550_prefixes,
 	.cmdline_usage = mitsu9550_cmdline,
 	.cmdline_arg = mitsu9550_cmdline_arg,
