@@ -1,6 +1,4 @@
 /*
- * "$Id: commandtoepson.c,v 1.4 2011/08/13 16:24:17 rlk Exp $"
- *
  *   EPSON ESC/P2 command filter for the Common UNIX Printing System.
  *
  *   Copyright 1993-2000 by Easy Software Products.
@@ -16,8 +14,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Contents:
  *
@@ -33,6 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <strings.h>
 
 /*
  * Macros...
@@ -227,8 +225,3 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   return (0);
 }
-
-
-/*
- * End of "$Id: commandtoepson.c,v 1.4 2011/08/13 16:24:17 rlk Exp $".
- */
