@@ -1,5 +1,4 @@
 /*
- * "$Id: escputil.c,v 1.110 2014/02/21 01:56:24 rlk Exp $"
  *
  *   Printer maintenance utility for EPSON Stylus (R) printers
  *
@@ -16,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,11 +25,13 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <strings.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include <ctype.h>
+#include <time.h>
 #if defined(HAVE_VARARGS_H) && !defined(HAVE_STDARG_H)
 #include <varargs.h>
 #else
@@ -90,8 +90,7 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License\n\
 for more details.\n\
 \n\
 You should have received a copy of the GNU General Public License\n\
-along with this program; if not, write to the Free Software\n\
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n");
+along with this program.  If not, see <https://www.gnu.org/licenses/>.\n");
 
 
 #if defined(HAVE_GETOPT_H) && defined(HAVE_GETOPT_LONG)
@@ -578,7 +577,8 @@ read_from_printer(int fd, char *buf, int bufsize, int quiet)
       status = read(fd, buf, bufsize - 1);
       if (status == 0 || (status < 0 && errno == EAGAIN))
 	{
-	  usleep(2000);
+	  struct timespec tm = {0, 2000000};
+	  nanosleep(&tm, &tm);
 	  status = 0; /* not an error (read would have blocked) */
 	}
     }
@@ -1092,6 +1092,13 @@ static const char *aux_colors[] =
     N_("Orange"),		/* d */
     NULL,			/* e */
     NULL,			/* f */
+    NULL,			/* 10 */
+    NULL,			/* 11 */
+    NULL,			/* 12 */
+    N_("Red"),			/* 13 */
+    NULL,			/* 14 */
+    NULL,			/* 15 */
+    N_("Red"),			/* 16 */
   };
 static int aux_color_count = sizeof(aux_colors) / sizeof(const char *);
 
@@ -1767,9 +1774,40 @@ do_extended_ink_info_1(const stp_printer_t *printer)
 		 gettext(stp_string_list_param(color_list, i)->text),
 		 val, id, (year > 80 ? 19 : 20), year, month);
 	}
+      else if (sscanf(ind,
+		      "IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:EPSON;",
+		      &val, &year, &month, &id ) == 4 ||
+	       sscanf(ind,
+		      "IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:INKbyEPSON;",
+		      &val, &year, &month, &id ) == 4)
+	{
+	  STP_DEBUG(printf("***Case 4: i %i val %ud year %ud mo %ud id %ud\n",
+			   i, val, year, month, id));
+	  if (i == 0)
+	    printf("%20s    %20s   %12s   %7s\n",
+		   _("Ink cartridge"), _("Percent remaining"), _("Part number"),
+		   _("Date"));
+	  printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
+		 gettext(stp_string_list_param(color_list, i)->text),
+		 val, id, (year > 80 ? 19 : 20), year, month);
+	}
+      else if (sscanf(ind,
+		      "II:%*2s;IQT:%x;PDY:%x;PDM:%x;STY:%*2s;STM:%*2s;STD:%*2s;EDY:%*2s;EDM:%*2s;EDD:%*2s;IC1:%x;IC2:%*4s;IK:%*4s;TOV:%*x;TVU:%*x;VIQ:%*4x;UIQ:%*4x;ERC:%*x;SID:%*x;LOG:EPSON   ;",
+		      &val, &year, &month, &id ) == 4)
+	{
+	  STP_DEBUG(printf("***Case 5: i %i val %ud year %ud mo %ud id %ud\n",
+			   i, val, year, month, id));
+	  if (i == 0)
+	    printf("%20s    %20s   %12s   %7s\n",
+		   _("Ink cartridge"), _("Percent remaining"), _("Part number"),
+		   _("Date"));
+	  printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
+		 gettext(stp_string_list_param(color_list, i)->text),
+		 val, id, (year > 80 ? 19 : 20), year, month);
+	}
       else
 	{
-	  STP_DEBUG(printf("***Case 5: failure %i (%s)\n", i, ind));
+	  STP_DEBUG(printf("***Case 6: failure %i (%s)\n", i, ind));
 	  printf("Cannot identify cartridge in slot %d\n", i);
 	}
     }
