@@ -1,5 +1,4 @@
 /*
- * "$Id: bit-ops.c,v 1.14 2009/06/07 15:21:22 rlk Exp $"
  *
  *   Softweave calculator for Gutenprint.
  *
@@ -16,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -36,6 +34,13 @@
 #include <limits.h>
 #endif
 
+#ifdef __GNUC__
+#define inline __inline__
+#define NOINLINE __attribute__ ((noinline))
+#else
+$define NOINLINE
+#endif
+
 void
 stp_fold(const unsigned char *line,
 	 int single_length,
@@ -279,6 +284,109 @@ stp_fold_4bit(const unsigned char *line,
     }
 }
 
+void
+stp_fold_8bit(const unsigned char *line,
+                int single_length,
+                unsigned char *outbuf)
+{
+  int i;
+  memset(outbuf, 0, single_length * 8);
+  for (i = 0; i < single_length; i++)
+    {
+      unsigned char l0 = line[0];
+      unsigned char l1 = line[single_length];
+      unsigned char l2 = line[single_length*2];
+      unsigned char l3 = line[single_length*3];
+      unsigned char l4 = line[single_length*4];
+      unsigned char l5 = line[single_length*5];
+      unsigned char l6 = line[single_length*6];
+      unsigned char l7 = line[single_length*7];
+      if (l0 || l1 || l2 || l3 || l4 || l5 || l6 || l7)
+	{
+	  outbuf[0] =		/* H7 G7 F7 E7 D7 C7 B7 A7 */
+            ((l7 & (1 << 7)) >> 0)|
+            ((l6 & (1 << 7)) >> 1)|
+            ((l5 & (1 << 7)) >> 2)|
+            ((l4 & (1 << 7)) >> 3)|
+            ((l3 & (1 << 7)) >> 4)|
+            ((l2 & (1 << 7)) >> 5)|
+            ((l1 & (1 << 7)) >> 6)|
+            ((l0 & (1 << 7)) >> 7);
+
+	  outbuf[1] =		/* H6 G6 F6 E6 D6 C6 B6 A6 */
+            ((l7 & (1 << 6)) << 1)|
+            ((l6 & (1 << 6)) >> 0)|
+            ((l5 & (1 << 6)) >> 1)|
+            ((l4 & (1 << 6)) >> 2)|
+            ((l3 & (1 << 6)) >> 3)|
+            ((l2 & (1 << 6)) >> 4)|
+            ((l1 & (1 << 6)) >> 5)|
+            ((l0 & (1 << 6)) >> 6);
+
+	  outbuf[2] =		/* H5 G5 F5 E5 D5 C5 B5 A5 */
+            ((l7 & (1 << 5)) << 2)|
+            ((l6 & (1 << 5)) << 1)|
+            ((l5 & (1 << 5)) >> 0)|
+            ((l4 & (1 << 5)) >> 1)|
+            ((l3 & (1 << 5)) >> 2)|
+            ((l2 & (1 << 5)) >> 3)|
+            ((l1 & (1 << 5)) >> 4)|
+            ((l0 & (1 << 5)) >> 5);
+
+	  outbuf[3] =		/* H4 G4 F4 E4 D4 C4 B4 A4 */
+            ((l7 & (1 << 4)) << 3)|
+            ((l6 & (1 << 4)) << 2)|
+            ((l5 & (1 << 4)) << 1)|
+            ((l4 & (1 << 4)) >> 0)|
+            ((l3 & (1 << 4)) >> 1)|
+            ((l2 & (1 << 4)) >> 2)|
+            ((l1 & (1 << 4)) >> 3)|
+            ((l0 & (1 << 4)) >> 4);
+	  outbuf[4] =		/* H3 G3 F3 E3 D3 C3 B3 A3 */
+            ((l7 & (1 << 3)) << 4)|
+            ((l6 & (1 << 3)) << 3)|
+            ((l5 & (1 << 3)) << 2)|
+            ((l4 & (1 << 3)) << 1)|
+            ((l3 & (1 << 3)) >> 0)|
+            ((l2 & (1 << 3)) >> 1)|
+            ((l1 & (1 << 3)) >> 2)|
+            ((l0 & (1 << 3)) >> 3);
+
+	  outbuf[5] =		/* H2 G2 F2 E2 D2 C2 B2 A2 */
+            ((l7 & (1 << 2)) << 5)|
+            ((l6 & (1 << 2)) << 4)|
+            ((l5 & (1 << 2)) << 3)|
+            ((l4 & (1 << 2)) << 2)|
+            ((l3 & (1 << 2)) << 1)|
+            ((l2 & (1 << 2)) >> 0)|
+            ((l1 & (1 << 2)) >> 1)|
+            ((l0 & (1 << 2)) >> 2);
+
+	  outbuf[6] =		/* H1 G1 F1 E1 D1 C1 B1 A1 */
+            ((l7 & (1 << 1)) << 6)|
+            ((l6 & (1 << 1)) << 5)|
+            ((l5 & (1 << 1)) << 4)|
+            ((l4 & (1 << 1)) << 3)|
+            ((l3 & (1 << 1)) << 2)|
+            ((l2 & (1 << 1)) << 1)|
+            ((l1 & (1 << 1)) >> 0)|
+            ((l0 & (1 << 1)) >> 1);
+
+	  outbuf[7] =		/* H0 G0 F0 E0 D0 C0 B0 A0 */
+            ((l7 & (1 << 0)) << 7)|
+            ((l6 & (1 << 0)) << 6)|
+            ((l5 & (1 << 0)) << 5)|
+            ((l4 & (1 << 0)) << 4)|
+            ((l3 & (1 << 0)) << 3)|
+            ((l2 & (1 << 0)) << 2)|
+            ((l1 & (1 << 0)) << 1)|
+            ((l0 & (1 << 0)) >> 0);
+	}
+      line++;
+      outbuf += 8;
+    }
+}
+
 #define SPLIT_MASK(k, b) (((1 << (b)) - 1) << ((k) * (b)))
 
 #define SPLIT_STEP(k, b, i, o, in, r, inc, rl)	\
@@ -375,7 +483,7 @@ stp_split_4(int length,
 }
 
 
-static void
+static void NOINLINE
 stpi_unpack_2_1(int length,
 		const unsigned char *in,
 		unsigned char **outs)
@@ -430,7 +538,7 @@ stpi_unpack_2_1(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_2_2(int length,
 		const unsigned char *in,
 		unsigned char **outs)
@@ -456,7 +564,7 @@ stpi_unpack_2_2(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_4_1(int length,
 		 const unsigned char *in,
 		 unsigned char **outs)
@@ -515,7 +623,7 @@ stpi_unpack_4_1(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_4_2(int length,
 		 const unsigned char *in,
 		 unsigned char **outs)
@@ -575,7 +683,7 @@ stpi_unpack_4_2(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_8_1(int length,
 		const unsigned char *in,
 		unsigned char **outs)
@@ -648,7 +756,7 @@ stpi_unpack_8_1(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_8_2(int length,
 		const unsigned char *in,
 		unsigned char **outs)
@@ -735,7 +843,7 @@ stpi_unpack_8_2(int length,
     }
 }
 
-static void
+static void NOINLINE
 stpi_unpack_16_1(int length,
 		 const unsigned char *in,
 		 unsigned char **outs)
@@ -806,7 +914,7 @@ stpi_unpack_16_1(int length,
       *outs[j]++ = temp[j];
 }
 
-static void
+static void NOINLINE
 stpi_unpack_16_2(int length,
 		 const unsigned char *in,
 		 unsigned char **outs)
@@ -1036,29 +1144,32 @@ stp_unpack_16(int length,
   stp_unpack(length, bits, 16, in, outs);
 }
 
-static void
+static void NOINLINE
 find_first_and_last(const unsigned char *line, int length,
 		    int *first, int *last)
 {
-  int i;
   int found_first = 0;
-  if (!first || !last)
-    return;
-  *first = 0;
-  *last = 0;
-  for (i = 0; i < length; i++)
+  int f = 0;
+  int l = 0;
+  for (f = 0; f < length; f++)
     {
-      if (line[i] == 0)
+      if (line[f])
 	{
-	  if (!found_first)
-	    (*first)++;
-	}
-      else
-	{
-	  *last = i;
 	  found_first = 1;
+	  break;
 	}
     }
+  *first = f;
+  if (!found_first)
+    {
+      *last = 0;
+      return;
+    }
+  for (l = length - 1; l >= f; l--)
+    if (line[l])
+      break;
+  ;
+  *last = l;
 }
 
 int
@@ -1088,96 +1199,89 @@ stp_pack_tiff(stp_vars_t *v,
 	      int *first,
 	      int *last)
 {
-  const unsigned char *start;		/* Start of compressed data */
-  unsigned char repeat;			/* Repeating char */
-  int count;			/* Count of compressed bytes */
-  int tcount;			/* Temporary count < 128 */
-  register const unsigned char *xline = line;
-  register int xlength = length;
-  find_first_and_last(line, length, first, last);
+  unsigned char *comp_pti = comp_buf;
+  if (first && last)
+    find_first_and_last(line, length, first, last);
 
   /*
    * Compress using TIFF "packbits" run-length encoding...
    */
 
-  (*comp_ptr) = comp_buf;
-
-  while (xlength > 0)
+  while (length > 0)
     {
+      const unsigned char *start = line;	/* Start of compressed data */
+      unsigned char repeat;		/* Repeating char */
+      int count;			/* Count of compressed bytes */
       /*
-       * Get a run of non-repeated chars...
+       * Get a run of at least 3 non-repeated chars...
        */
 
-      start  = xline;
-      xline   += 2;
-      xlength -= 2;
+      line   += 2;
+      length -= 2;
 
-      while (xlength > 0 && (xline[-2] != xline[-1] || xline[-1] != xline[0]))
+      while (length > 0 && (line[-2] != line[-1] || line[-1] != line[0]))
 	{
-	  xline ++;
-	  xlength --;
+	  line ++;
+	  length --;
 	}
 
-      xline   -= 2;
-      xlength += 2;
+      line   -= 2;
+      length += 2;
 
       /*
        * Output the non-repeated sequences (max 128 at a time).
        */
 
-      count = xline - start;
+      count = line - start;
       while (count > 0)
 	{
-	  tcount = count > 128 ? 128 : count;
+	  int tcount = count > 128 ? 128 : count;
 
-	  (*comp_ptr)[0] = tcount - 1;
-	  memcpy((*comp_ptr) + 1, start, tcount);
+	  comp_pti[0] = tcount - 1;
+	  memcpy(comp_pti + 1, start, tcount);
 
-	  (*comp_ptr) += tcount + 1;
+	  comp_pti += tcount + 1;
 	  start    += tcount;
 	  count    -= tcount;
 	}
 
-      if (xlength <= 0)
+      if (length <= 0)
 	break;
 
       /*
        * Find the repeated sequences...
        */
 
-      start  = xline;
-      repeat = xline[0];
+      start  = line;
+      repeat = line[0];
 
-      xline ++;
-      xlength --;
+      line ++;
+      length --;
 
-      if (xlength > 0)
+      while (length > 0 && *line == repeat)
 	{
-	  int ylength = xlength;
-	  while (ylength && *xline == repeat)
-	    {
-	      xline ++;
-	      ylength --;
-	    }
-	  xlength = ylength;
+	  line++;
+	  length--;
 	}
 
       /*
        * Output the repeated sequences (max 128 at a time).
        */
 
-      count = xline - start;
+      count = line - start;
       while (count > 0)
 	{
-	  tcount = count > 128 ? 128 : count;
+	  int tcount = count > 128 ? 128 : count;
 
-	  (*comp_ptr)[0] = 1 - tcount;
-	  (*comp_ptr)[1] = repeat;
+	  comp_pti[0] = 1 - tcount;
+	  comp_pti[1] = repeat;
 
-	  (*comp_ptr) += 2;
+	  comp_pti += 2;
 	  count    -= tcount;
 	}
     }
+  (*comp_ptr) = comp_pti;
+
   if (first && last && *first > *last)
     return 0;
   else
