@@ -1,5 +1,4 @@
 /*
- * "$Id: curve.c,v 1.54 2008/10/29 00:10:42 easysw Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -17,8 +16,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +35,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <strings.h>
 
 #ifdef __GNUC__
 #define inline __inline__
@@ -45,6 +44,16 @@
 #undef inline
 #define inline
 
+/*
+ * We could do more sanity checks here if we want.
+ */
+#define CHECK_CURVE(curve)			\
+do						\
+  {						\
+    STPI_ASSERT((curve) != NULL, NULL);		\
+    STPI_ASSERT((curve)->seq != NULL, NULL);	\
+  } while (0)
+
 static const int curve_point_limit = 1048576;
 
 struct stp_curve
@@ -75,27 +84,6 @@ static const char *const stpi_wrap_mode_names[] =
     "wrap"
   };
 
-static const int stpi_wrap_mode_count =
-(sizeof(stpi_wrap_mode_names) / sizeof(const char *));
-
-/*
- * We could do more sanity checks here if we want.
- */
-static void
-check_curve(const stp_curve_t *curve)
-{
-  if (curve == NULL)
-    {
-      stp_erprintf("Null curve! Please report this bug.\n");
-      stp_abort();
-    }
-  if (curve->seq == NULL)
-    {
-      stp_erprintf("Bad curve (seq == NULL)! Please report this bug.\n");
-      stp_abort();
-    }
-}
-
 /*
  * Get the total number of points in the base sequence class
  */
@@ -397,6 +385,25 @@ stpi_curve_set_points(stp_curve_t *curve, size_t points)
   return 1;
 }
 
+static int
+stpi_curve_set_data(stp_curve_t *curve, size_t points, const double *data)
+{
+  if (points < 2)
+    return 0;
+  if (points > curve_point_limit ||
+      (curve->wrap_mode == STP_CURVE_WRAP_AROUND &&
+       points > curve_point_limit - 1))
+    return 0;
+  clear_curve_data(curve);
+  if (curve->wrap_mode == STP_CURVE_WRAP_AROUND)
+    points++;
+  if (curve->piecewise)
+    points *= 2;
+  if ((stp_sequence_set_data(curve->seq, points, data)) == 0)
+    return 0;
+  return 1;
+}
+
 /*
  * Create a default curve
  */
@@ -430,7 +437,7 @@ stp_curve_create(stp_curve_wrap_mode_t wrap_mode)
 static void
 curve_dtor(stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   clear_curve_data(curve);
   if (curve->seq)
     stp_sequence_destroy(curve->seq);
@@ -448,8 +455,8 @@ stp_curve_destroy(stp_curve_t *curve)
 void
 stp_curve_copy(stp_curve_t *dest, const stp_curve_t *source)
 {
-  check_curve(dest);
-  check_curve(source);
+  CHECK_CURVE(dest);
+  CHECK_CURVE(source);
   curve_dtor(dest);
   dest->curve_type = source->curve_type;
   dest->wrap_mode = source->wrap_mode;
@@ -463,7 +470,7 @@ stp_curve_t *
 stp_curve_create_copy(const stp_curve_t *curve)
 {
   stp_curve_t *ret;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   ret = stp_curve_create(curve->wrap_mode);
   stp_curve_copy(ret, curve);
   return ret;
@@ -472,8 +479,8 @@ stp_curve_create_copy(const stp_curve_t *curve)
 void
 stp_curve_reverse(stp_curve_t *dest, const stp_curve_t *source)
 {
-  check_curve(dest);
-  check_curve(source);
+  CHECK_CURVE(dest);
+  CHECK_CURVE(source);
   curve_dtor(dest);
   dest->curve_type = source->curve_type;
   dest->wrap_mode = source->wrap_mode;
@@ -506,7 +513,7 @@ stp_curve_t *
 stp_curve_create_reverse(const stp_curve_t *curve)
 {
   stp_curve_t *ret;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   ret = stp_curve_create(curve->wrap_mode);
   stp_curve_reverse(ret, curve);
   return ret;
@@ -515,14 +522,14 @@ stp_curve_create_reverse(const stp_curve_t *curve)
 int
 stp_curve_set_bounds(stp_curve_t *curve, double low, double high)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return stp_sequence_set_bounds(curve->seq, low, high);
 }
 
 void
 stp_curve_get_bounds(const stp_curve_t *curve, double *low, double *high)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   stp_sequence_get_bounds(curve->seq, low, high);
 }
 
@@ -535,36 +542,36 @@ stp_curve_get_bounds(const stp_curve_t *curve, double *low, double *high)
 void
 stp_curve_get_range(const stp_curve_t *curve, double *low, double *high)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   stp_sequence_get_range(curve->seq, low, high);
 }
 
 size_t
 stp_curve_count_points(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return get_point_count(curve);
 }
 
 stp_curve_wrap_mode_t
 stp_curve_get_wrap(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return curve->wrap_mode;
 }
 
 int
 stp_curve_is_piecewise(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return curve->piecewise;
 }
 
 int
 stp_curve_set_interpolation_type(stp_curve_t *curve, stp_curve_type_t itype)
 {
-  check_curve(curve);
-  if (itype < 0 || itype >= stpi_curve_type_count)
+  CHECK_CURVE(curve);
+  if (itype >= stpi_curve_type_count)
     return 0;
   curve->curve_type = itype;
   return 1;
@@ -573,14 +580,14 @@ stp_curve_set_interpolation_type(stp_curve_t *curve, stp_curve_type_t itype)
 stp_curve_type_t
 stp_curve_get_interpolation_type(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return curve->curve_type;
 }
 
 int
 stp_curve_set_gamma(stp_curve_t *curve, double fgamma)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (curve->wrap_mode || ! isfinite(fgamma) || fgamma == 0.0)
     return 0;
   clear_curve_data(curve);
@@ -592,7 +599,7 @@ stp_curve_set_gamma(stp_curve_t *curve, double fgamma)
 double
 stp_curve_get_gamma(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   return curve->gamma;
 }
 
@@ -602,7 +609,7 @@ stp_curve_set_data(stp_curve_t *curve, size_t count, const double *data)
   size_t i;
   size_t real_count = count;
   double low, high;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (count < 2)
     return 0;
   if (curve->wrap_mode == STP_CURVE_WRAP_AROUND)
@@ -642,7 +649,7 @@ stp_curve_set_data_points(stp_curve_t *curve, size_t count,
   size_t real_count = count;
   double low, high;
   double last_x = -1;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (count < 2)
     {
       stp_deprintf(STP_DBG_CURVE_ERRORS,
@@ -691,7 +698,7 @@ stp_curve_set_data_points(stp_curve_t *curve, size_t count,
 		       "stp_curve_set_data_points: horizontal value must "
 		       "not exceed .99999\n");
 	  return 0;
-	}	  
+	}
       if (data[i].x < 0 || data[i].x > 1)
 	{
 	  stp_deprintf(STP_DBG_CURVE_ERRORS,
@@ -736,7 +743,7 @@ const double *
 stp_curve_get_data(const stp_curve_t *curve, size_t *count)
 {
   const double *ret;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (curve->piecewise)
     return NULL;
   stp_sequence_get_data(curve->seq, count, &ret);
@@ -748,7 +755,7 @@ const stp_curve_point_t *
 stp_curve_get_data_points(const stp_curve_t *curve, size_t *count)
 {
   const double *ret;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (!curve->piecewise)
     return NULL;
   stp_sequence_get_data(curve->seq, count, &ret);
@@ -760,7 +767,7 @@ static const double *
 stpi_curve_get_data_internal(const stp_curve_t *curve, size_t *count)
 {
   const double *ret;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   stp_sequence_get_data(curve->seq, count, &ret);
   *count = get_point_count(curve);
   if (curve->piecewise)
@@ -780,7 +787,7 @@ stp_curve_set_##name##_data(stp_curve_t *curve, size_t count, const t *data) \
   int status;                                                              \
   size_t real_count = count;                                               \
                                                                            \
-  check_curve(curve);                                                      \
+  CHECK_CURVE(curve);                                                      \
   if (count < 2)                                                           \
     return 0;                                                              \
   if (curve->wrap_mode == STP_CURVE_WRAP_AROUND)                           \
@@ -853,7 +860,7 @@ stp_curve_set_subrange(stp_curve_t *curve, const stp_curve_t *range,
   double rlo, rhi;
   const double *data;
   size_t count;
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (start + stp_curve_count_points(range) > stp_curve_count_points(curve))
     return 0;
   if (curve->piecewise)
@@ -875,7 +882,7 @@ stp_curve_set_subrange(stp_curve_t *curve, const stp_curve_t *range,
 int
 stp_curve_set_point(stp_curve_t *curve, size_t where, double data)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (where >= get_point_count(curve))
     return 0;
   curve->gamma = 0.0;
@@ -895,7 +902,7 @@ stp_curve_set_point(stp_curve_t *curve, size_t where, double data)
 int
 stp_curve_get_point(const stp_curve_t *curve, size_t where, double *data)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (where >= get_point_count(curve))
     return 0;
   if (curve->piecewise)
@@ -906,7 +913,7 @@ stp_curve_get_point(const stp_curve_t *curve, size_t where, double *data)
 const stp_sequence_t *
 stp_curve_get_sequence(const stp_curve_t *curve)
 {
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (curve->piecewise)
     return NULL;
   return curve->seq;
@@ -916,15 +923,12 @@ int
 stp_curve_rescale(stp_curve_t *curve, double scale,
 		  stp_curve_compose_t mode, stp_curve_bounds_t bounds_mode)
 {
-  size_t real_point_count;
   int i;
   double nblo;
   double nbhi;
   size_t count;
 
-  check_curve(curve);
-
-  real_point_count = get_real_point_count(curve);
+  CHECK_CURVE(curve);
 
   stp_sequence_get_bounds(curve->seq, &nblo, &nbhi);
   if (bounds_mode == STP_CURVE_BOUNDS_RESCALE)
@@ -1045,7 +1049,7 @@ interpolate_gamma_internal(const stp_curve_t *curve, double where)
   double blo, bhi;
   size_t real_point_count;
 
-  real_point_count = get_real_point_count(curve);;
+  real_point_count = get_real_point_count(curve);
 
   if (real_point_count)
     where /= (real_point_count - 1);
@@ -1068,7 +1072,7 @@ do_interpolate_spline(double low, double high, double frac,
 {
   double a = 1.0 - frac;
   double b = frac;
-  double retval = 
+  double retval =
     ((a * a * a - a) * interval_low) + ((b * b * b - b) * interval_high);
   retval = retval * x_interval * x_interval / 6;
   retval += (a * low) + (b * high);
@@ -1076,7 +1080,7 @@ do_interpolate_spline(double low, double high, double frac,
 }
 
 static inline double
-interpolate_point_internal(stp_curve_t *curve, double where)
+interpolate_point_internal(const stp_curve_t *curve, double where)
 {
   int integer = where;
   double frac = where - (double) integer;
@@ -1090,7 +1094,7 @@ interpolate_point_internal(stp_curve_t *curve, double where)
       return val;
     }
   if (curve->recompute_interval)
-    compute_intervals(curve);
+    compute_intervals((stpi_cast_safe(curve)));
   if (curve->curve_type == STP_CURVE_TYPE_LINEAR)
     {
       double val;
@@ -1133,7 +1137,7 @@ stp_curve_interpolate_value(const stp_curve_t *curve, double where,
 {
   size_t limit;
 
-  check_curve(curve);
+  CHECK_CURVE(curve);
   if (curve->piecewise)
     return 0;
 
@@ -1144,7 +1148,7 @@ stp_curve_interpolate_value(const stp_curve_t *curve, double where,
   if (curve->gamma)	/* this means a pure gamma curve */
     *result = interpolate_gamma_internal(curve, where);
   else
-    *result = interpolate_point_internal((stp_curve_t *) curve, where);
+    *result = interpolate_point_internal(curve, where);
   return 1;
 }
 
@@ -1156,7 +1160,7 @@ stp_curve_resample(stp_curve_t *curve, size_t points)
   size_t i;
   double *new_vec;
 
-  check_curve(curve);
+  CHECK_CURVE(curve);
 
   if (points == get_point_count(curve) && curve->seq && !(curve->piecewise))
     return 1;
@@ -1246,20 +1250,61 @@ stp_curve_resample(stp_curve_t *curve, size_t points)
 	}
       curve->piecewise = 0;
     }
+  else if (curve->gamma)
+    {
+      double fgamma = curve->gamma;
+      double blo, bhi;
+      int negative_gamma = 0;
+      stp_sequence_get_bounds(curve->seq, &blo, &bhi);
+      if (fgamma > 0)
+	{
+	  fgamma = -fgamma;
+	  negative_gamma = 1;
+	}
+      for (i = 0; i < limit; i++)
+	{
+	  double where = ((double) i * (double) old / (double) (limit - 1));
+	  if (negative_gamma)
+	    where = 1.0 - where;
+	  new_vec[i] = blo + ((bhi - blo) * pow(where, fgamma));
+	}
+    }
   else
     {
+      double blo, bhi;
+      const double *seq_data;
+      size_t seq_count;
+      size_t point_count = get_point_count(curve);
+      stp_sequence_get_data(curve->seq, &seq_count, &seq_data);
+      stp_sequence_get_bounds(curve->seq, &blo, &bhi);
+	if (curve->recompute_interval)
+	  compute_intervals((stpi_cast_safe(curve)));
       for (i = 0; i < limit; i++)
-	if (curve->gamma)
-	  new_vec[i] =
-	    interpolate_gamma_internal(curve, ((double) i * (double) old /
-					       (double) (limit - 1)));
-	else
-	  new_vec[i] =
-	    interpolate_point_internal(curve, ((double) i * (double) old /
-					       (double) (limit - 1)));
+	{
+	  double where = ((double) i * (double) old / (double) (limit - 1));
+	  int iwhere = (int) where;
+	  double frac = where - (double) iwhere;
+	  if (frac == 0.0)
+	    new_vec[i] = seq_data[iwhere];
+	  else if (curve->curve_type == STP_CURVE_TYPE_LINEAR)
+	    new_vec[i] = seq_data[iwhere] + (frac * curve->interval[iwhere]);
+	  else
+	    {
+	      int iwhere1 = iwhere + 1;
+	      while (iwhere1 > point_count)
+		iwhere1 -= point_count;
+	      new_vec[i] =
+		do_interpolate_spline(seq_data[iwhere], seq_data[iwhere1],
+				      frac, curve->interval[iwhere],
+				      curve->interval[iwhere1], 1.0);
+	      if (new_vec[i] > bhi)
+		new_vec[i] = bhi;
+	      else if (new_vec[i] < blo)
+		new_vec[i] = blo;
+	    }
+	}
     }
-  stpi_curve_set_points(curve, points);
-  stp_sequence_set_subrange(curve->seq, 0, limit, new_vec);
+  stpi_curve_set_data(curve, points, new_vec);
   curve->recompute_interval = 1;
   stp_free(new_vec);
   return 1;
@@ -1461,6 +1506,9 @@ stp_curve_create_from_xmltree(stp_mxml_node_t *curve)  /* The curve node */
   int piecewise = 0;
 
   stp_xml_init();
+  /* FIXME Need protection against unlimited recursion */
+  if ((stmp = stp_mxmlElementGetAttr(curve, "src")) != NULL)
+    return stp_curve_create_from_file(stmp);
   /* Get curve type */
   stmp = stp_mxmlElementGetAttr(curve, "type");
   if (stmp)
@@ -1532,7 +1580,7 @@ stp_curve_create_from_xmltree(stp_mxml_node_t *curve)  /* The curve node */
   ret = stp_curve_create(wrap_mode);
   stp_curve_set_interpolation_type(ret, curve_type);
 
-  child = stp_mxmlFindElement(curve, curve, "sequence", NULL, NULL, STP_MXML_DESCEND);
+  child = stp_xml_get_node(curve, "sequence", NULL);
   if (child)
     seq = stp_sequence_create_from_xmltree(child);
 
@@ -1604,6 +1652,11 @@ stp_curve_create_from_xmltree(stp_mxml_node_t *curve)  /* The curve node */
  error:
   stp_deprintf(STP_DBG_CURVE_ERRORS,
 	       "stp_curve_create_from_xmltree: error during curve read\n");
+  if (seq)
+    {
+      stp_sequence_destroy(seq);
+      seq = NULL;
+    }
   if (ret)
     stp_curve_destroy(ret);
   stp_xml_exit();
@@ -1876,7 +1929,20 @@ stp_curve_create_from_file(const char* file)
 {
   stp_curve_t *curve = NULL;
   stp_mxml_node_t *doc;
-  FILE *fp = fopen(file, "r");
+  FILE *fp = NULL;
+  if (file[0] != '/' && strncmp(file, "./", 2) && strncmp(file, "../", 3))
+    {
+      char *fn = stp_path_find_file(NULL, file);
+      if (fn)
+	{
+	  fp = fopen(file, "r");
+	  free(fn);
+	}
+    }
+  else if (file)
+    {
+      fp = fopen(file, "r");
+    }
   if (!fp)
     {
       stp_deprintf(STP_DBG_CURVE_ERRORS,
