@@ -1,5 +1,4 @@
 /*
- * "$Id: escp2-driver.c,v 1.54 2008/07/26 23:15:47 rlk Exp $"
  *
  *   Print plug-in EPSON ESC/P2 driver for the GIMP.
  *
@@ -17,8 +16,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -54,7 +52,7 @@ escp2_reset_printer(stp_vars_t *v)
    * packet mode.
    */
   if (pd->preinit_sequence)
-    stp_zfwrite(pd->preinit_sequence->data, pd->preinit_sequence->bytes, 1, v);
+    stp_write_raw(pd->preinit_sequence, v);
 
   stp_send_command(v, "\033@", "");
 }
@@ -83,6 +81,12 @@ print_remote_float_param(stp_vars_t *v, const char *param, double value)
   print_remote_param(v, param, buf);
 }
 
+static void
+print_remote_dim_param(stp_vars_t *v, const char *param, double value)
+{
+  print_remote_float_param(v, param, value);
+}
+
 static void
 print_debug_params(stp_vars_t *v)
 {
@@ -94,10 +98,10 @@ print_debug_params(stp_vars_t *v)
   print_remote_param(v, "Version", VERSION);
   print_remote_param(v, "Release Date", RELEASE_DATE);
   print_remote_param(v, "Driver", stp_get_driver(v));
-  print_remote_int_param(v, "Left", stp_get_left(v));
-  print_remote_int_param(v, "Top", stp_get_top(v));
-  print_remote_int_param(v, "Page Width", stp_get_page_width(v));
-  print_remote_int_param(v, "Page Height", stp_get_page_height(v));
+  print_remote_dim_param(v, "Left", stp_get_left(v));
+  print_remote_dim_param(v, "Top", stp_get_top(v));
+  print_remote_dim_param(v, "Page Width", stp_get_page_width(v));
+  print_remote_dim_param(v, "Page Height", stp_get_page_height(v));
   print_remote_int_param(v, "Model", stp_get_model_id(v));
   print_remote_int_param(v, "Ydpi", pd->res->vres);
   print_remote_int_param(v, "Xdpi", pd->res->hres);
@@ -109,19 +113,23 @@ print_debug_params(stp_vars_t *v)
 */
   print_remote_int_param(v, "Use_printer_weave", pd->use_printer_weave);
   print_remote_int_param(v, "Duplex", pd->duplex);
-  print_remote_int_param(v, "Page_left", pd->page_left);
-  print_remote_int_param(v, "Page_right", pd->page_right);
-  print_remote_int_param(v, "Page_top", pd->page_top);
-  print_remote_int_param(v, "Page_bottom", pd->page_bottom);
-  print_remote_int_param(v, "Page_width", pd->page_width);
-  print_remote_int_param(v, "Page_height", pd->page_height);
-  print_remote_int_param(v, "Page_true_height", pd->page_true_height);
-  print_remote_int_param(v, "Page_extra_height", pd->page_extra_height);
-  print_remote_int_param(v, "Paper_extra_bottom", pd->paper_extra_bottom);
-  print_remote_int_param(v, "Image_left", pd->image_left);
-  print_remote_int_param(v, "Image_top", pd->image_top);
-  print_remote_int_param(v, "Image_width", pd->image_width);
-  print_remote_int_param(v, "Image_height", pd->image_height);
+  print_remote_dim_param(v, "Page_left", pd->page_left);
+  print_remote_dim_param(v, "Page_right", pd->page_right);
+  print_remote_dim_param(v, "Page_top", pd->page_top);
+  print_remote_dim_param(v, "Page_bottom", pd->page_bottom);
+  print_remote_dim_param(v, "Page_width", pd->page_width);
+  print_remote_dim_param(v, "Page_height", pd->page_height);
+  print_remote_dim_param(v, "Page_true_height", pd->page_true_height);
+  print_remote_dim_param(v, "Page_extra_height", pd->page_extra_height);
+  print_remote_dim_param(v, "Paper_extra_bottom", pd->paper_extra_bottom);
+  print_remote_dim_param(v, "Image_left", pd->image_left);
+  print_remote_dim_param(v, "Image_top", pd->image_top);
+  print_remote_dim_param(v, "Image_width", pd->image_width);
+  print_remote_dim_param(v, "Image_height", pd->image_height);
+  print_remote_dim_param(v, "CD_X_offset", pd->cd_x_offset);
+  print_remote_dim_param(v, "CD_Y_offset", pd->cd_y_offset);
+  print_remote_dim_param(v, "CD_inner_radius", pd->cd_inner_radius);
+  print_remote_dim_param(v, "CD_outer_radius", pd->cd_outer_radius);
   print_remote_int_param(v, "Image_scaled_width", pd->image_scaled_width);
   print_remote_int_param(v, "Image_scaled_height", pd->image_scaled_height);
   print_remote_int_param(v, "Image_printed_width", pd->image_printed_width);
@@ -226,24 +234,20 @@ escp2_set_remote_sequence(stp_vars_t *v)
 	  pd->input_slot->roll_feed_cut_flags == ROLL_FEED_CUT_ALL)
 	      stp_send_command(v, "JS", "bh", 0);
       if (pd->preinit_remote_sequence)
-	stp_zfwrite(pd->preinit_remote_sequence->data,
-		    pd->preinit_remote_sequence->bytes, 1, v);
+	stp_write_raw(pd->preinit_remote_sequence, v);
       if (stp_check_int_parameter(pv, "FeedAdjustment", STP_PARAMETER_ACTIVE))
 	stp_send_command(v, "SN", "bccc", 0, 4,
 			 stp_get_int_parameter(pv, "FeedAdjustment"));
       if (stp_check_int_parameter(pv, "VacuumIntensity", STP_PARAMETER_ACTIVE))
 	stp_send_command(v, "SN", "bccc", 0, 5,
 			 stp_get_int_parameter(pv, "VacuumIntensity"));
-      if (stp_check_int_parameter(pv, "ScanDryTime", STP_PARAMETER_ACTIVE))
-	stp_send_command(v, "DR", "bcch", 0, 1,
-			 (int) stp_get_float_parameter(pv, "ScanDryTime") * 1000);
-      if (stp_check_int_parameter(pv, "ScanMinDryTime", STP_PARAMETER_ACTIVE))
-	{
-	  stp_send_command(v, "DR", "bcccc", 0, 0x41, 0xff, 0xff);
-	  stp_send_command(v, "DR", "bcch", 0, 1,
-			   (int) stp_get_float_parameter(pv, "ScanMinDryTime") * 1000);
-	}
-      if (stp_check_int_parameter(pv, "PageDryTime", STP_PARAMETER_ACTIVE))
+      if (stp_check_float_parameter(pv, "ScanDryTime", STP_PARAMETER_ACTIVE))
+	stp_send_command(v, "DR", "bcch", 0, 0,
+			 (int) (stp_get_float_parameter(pv, "ScanDryTime") * 1000));
+      if (stp_check_float_parameter(pv, "ScanMinDryTime", STP_PARAMETER_ACTIVE))
+	stp_send_command(v, "DR", "bcch", 0, 0x40,
+			 (int) (stp_get_float_parameter(pv, "ScanMinDryTime") * 1000));
+      if (stp_check_float_parameter(pv, "PageDryTime", STP_PARAMETER_ACTIVE))
 	stp_send_command(v, "DR", "bcch", 0, 1,
 			 (int) stp_get_float_parameter(pv, "PageDryTime"));
       /* Next comes paper path */
@@ -252,8 +256,7 @@ escp2_set_remote_sequence(stp_vars_t *v)
 	  int divisor = pd->base_separation / 360;
 	  int height = pd->page_true_height * 5 / divisor;
 	  if (pd->input_slot->init_sequence)
-	    stp_zfwrite(pd->input_slot->init_sequence->data,
-			pd->input_slot->init_sequence->bytes, 1, v);
+	    stp_write_raw(pd->input_slot->init_sequence, v);
 	  switch (pd->input_slot->roll_feed_cut_flags)
 	    {
 	    case ROLL_FEED_CUT_ALL:
@@ -278,7 +281,7 @@ escp2_set_remote_sequence(stp_vars_t *v)
 	{
 	  /* If there's ever duplex no tumble, we'll need to special
 	     case it, too */
-	  if (pd->duplex == DUPLEX_TUMBLE && (pd->input_slot->duplex & DUPLEX_TUMBLE))
+	  if (pd->duplex == DUPLEX_TUMBLE && pd->input_slot && (pd->input_slot->duplex & DUPLEX_TUMBLE))
 	    stp_send_command(v, "DP", "bcc", 0, 2); /* Auto duplex */
 	  else
 	    stp_send_command(v, "DP", "bcc", 0, 2); /* Auto duplex */
@@ -297,10 +300,10 @@ escp2_set_remote_sequence(stp_vars_t *v)
 	  stp_send_command(v, "FP", "bch", 0,
 			   (unsigned short) -pd->zero_margin_offset);
 	  if (pd->borderless_sequence)
-	    stp_zfwrite(pd->borderless_sequence->data,
-			pd->borderless_sequence->bytes,
-			1, v);
+	    stp_write_raw(pd->borderless_sequence, v);
 	}
+      if (pd->inkname->init_sequence)
+	stp_write_raw(pd->inkname->init_sequence, v);
       /* Exit remote mode */
 
       stp_send_command(v, "\033", "ccc", 0, 0, 0);
@@ -344,7 +347,7 @@ escp2_set_printer_weave(stp_vars_t *v)
 {
   escp2_privdata_t *pd = get_privdata(v);
   if (pd->printer_weave)
-    stp_zfwrite(pd->printer_weave->data, pd->printer_weave->bytes, 1, v);
+    stp_write_raw(pd->printer_weave, v);
   else
     stp_send_command(v, "\033(i", "bc", 0);
 }
@@ -444,7 +447,7 @@ escp2_set_paper_dimensions(stp_vars_t *v)
 	pd->page_management_units / 72;
       stp_send_command(v, "\033(S", "bll", w, h);
       if (stp_check_int_parameter(pv, "PrintMethod", STP_PARAMETER_ACTIVE))
-	stp_send_command(v, "\033(m", "bc", 
+	stp_send_command(v, "\033(m", "bc",
 			 stp_get_int_parameter(pv, "PrintMethod"));
     }
 }
@@ -634,16 +637,16 @@ stpi_escp2_deinit_printer(stp_vars_t *v)
   if (pd->advanced_command_set || pd->input_slot)
     {
       stp_send_command(v, "\033(R", "bcs", 0, "REMOTE1");
+      if (pd->inkname->deinit_sequence)
+	stp_write_raw(pd->inkname->deinit_sequence, v);
       if (pd->input_slot && pd->input_slot->deinit_sequence)
-	stp_zfwrite(pd->input_slot->deinit_sequence->data,
-		    pd->input_slot->deinit_sequence->bytes, 1, v);
+	stp_write_raw(pd->input_slot->deinit_sequence, v);
       /* Load settings from NVRAM */
       stp_send_command(v, "LD", "b");
 
       /* Magic deinit sequence reported by Simone Falsini */
       if (pd->deinit_remote_sequence)
-	stp_zfwrite(pd->deinit_remote_sequence->data,
-		    pd->deinit_remote_sequence->bytes, 1, v);
+	stp_write_raw(pd->deinit_remote_sequence, v);
       /* Exit remote mode */
       stp_send_command(v, "\033", "ccc", 0, 0, 0);
     }
