@@ -1,5 +1,4 @@
 /*
- * "$Id: print-dither-matrices.c,v 1.38 2010/08/04 00:33:56 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -16,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Revision History:
  *
@@ -34,6 +32,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include "dither-impl.h"
+#include <sys/param.h>
 
 #ifdef __GNUC__
 #define inline __inline__
@@ -524,7 +523,7 @@ stp_xml_dither_cache_set(int x, int y, const char *filename)
 static int
 stp_xml_process_dither_matrix(stp_mxml_node_t *dm,     /* The dither matrix node */
 			       const char *file)  /* Source file */
-			       
+
 {
   const char *value;
   int x = -1;
@@ -544,7 +543,7 @@ stp_xml_process_dither_matrix(stp_mxml_node_t *dm,     /* The dither matrix node
 }
 
 static stp_array_t *
-stpi_dither_array_create_from_xmltree(stp_mxml_node_t *dm) /* Dither matrix node */
+stpi_dither_array_create_from_xmltree(stp_mxml_node_t *dm, int x, int y) /* Dither matrix node */
 {
   const char *stmp;
   stp_mxml_node_t *child;
@@ -573,8 +572,15 @@ stpi_dither_array_create_from_xmltree(stp_mxml_node_t *dm) /* Dither matrix node
       goto error;
     }
 
+  if (x * y_aspect != y * x_aspect)
+    {
+      stp_erprintf("stpi_dither_array_create_from_xmltree: requested aspect of (%d, %d), found (%d, %d)\n",
+		   x, y, x_aspect, y_aspect);
+      goto error;
+    }
+
   /* Now read in the array */
-  child = stp_mxmlFindElement(dm, dm, "array", NULL, NULL, STP_MXML_DESCEND);
+  child = stp_xml_get_node(dm, "array", NULL);
   if (child)
     return stp_array_create_from_xmltree(child);
   else
@@ -584,7 +590,7 @@ stpi_dither_array_create_from_xmltree(stp_mxml_node_t *dm) /* Dither matrix node
 }
 
 static stp_array_t *
-xml_doc_get_dither_array(stp_mxml_node_t *doc)
+xml_doc_get_dither_array(stp_mxml_node_t *doc, int x, int y)
 {
   stp_mxml_node_t *cur;
   stp_mxml_node_t *xmlseq;
@@ -610,11 +616,11 @@ xml_doc_get_dither_array(stp_mxml_node_t *doc)
       return NULL;
     }
 
-  return stpi_dither_array_create_from_xmltree(xmlseq);
+  return stpi_dither_array_create_from_xmltree(xmlseq, x, y);
 }
 
 static stp_array_t *
-stpi_dither_array_create_from_file(const char* file)
+stpi_dither_array_create_from_file(const char* file, int x, int y)
 {
   stp_mxml_node_t *doc;
   stp_array_t *ret = NULL;
@@ -637,7 +643,7 @@ stpi_dither_array_create_from_file(const char* file)
 
   if (doc)
     {
-      ret = xml_doc_get_dither_array(doc);
+      ret = xml_doc_get_dither_array(doc, x, y);
       stp_mxmlDelete(doc);
     }
 
@@ -659,8 +665,8 @@ stp_xml_get_dither_array(int x, int y)
 
   if (!cachedval)
     {
-      char buf[1024];
-      (void) sprintf(buf, "dither-matrix-%dx%d.xml", x, y);
+      char buf[MAXPATHLEN+1];
+      (void) snprintf(buf, MAXPATHLEN, "dither/matrix-%dx%d.xml", x, y);
       stp_xml_parse_file_named(buf);
       cachedval = stp_xml_dither_cache_get(x, y);
       if (cachedval == NULL || cachedval->filename == NULL)
@@ -669,7 +675,7 @@ stp_xml_get_dither_array(int x, int y)
 	}
     }
 
-  ret = stpi_dither_array_create_from_file(cachedval->filename);
+  ret = stpi_dither_array_create_from_file(cachedval->filename, x, y);
 
   cachedval->dither_array = ret;
   return stp_array_create_copy(ret);
@@ -694,7 +700,7 @@ stp_find_standard_dither_array(int x_aspect, int y_aspect)
     x_aspect += 1;		/* so cheat */
   if (y_aspect == 3)
     y_aspect += 1;
-  
+
   divisor = gcd(x_aspect, y_aspect);
   x_aspect /= divisor;
   y_aspect /= divisor;
