@@ -1,6 +1,4 @@
 /*
- * "$Id: print-escp2.h,v 1.139 2010/12/19 02:51:37 rlk Exp $"
- *
  *   Print plug-in EPSON ESC/P2 driver for the GIMP.
  *
  *   Copyright 1997-2000 Michael Sweet (mike@easysw.com) and
@@ -17,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef GUTENPRINT_INTERNAL_ESCP2_H
@@ -28,8 +25,8 @@
  * Maximum number of channels in a printer.  If Epson comes out with an
  * 8-head printer, this needs to be increased.
  */
-#define PHYSICAL_CHANNEL_LIMIT 8
 #define MAX_DROP_SIZES 3
+#define PHYSICAL_CHANNEL_LIMIT ((STP_CHANNEL_LIMIT) / ((MAX_DROP_SIZES)+1))
 
 #define XCOLOR_R     (STP_NCOLORS + 0)
 #define XCOLOR_B     (STP_NCOLORS + 1)
@@ -224,7 +221,7 @@ typedef struct
   short n_inklists;
   inklist_t *inklists;
 } inkgroup_t;
-    
+
 
 /*
  ****************************************************************
@@ -380,48 +377,48 @@ typedef struct escp2_printer
 					       /* in auto mode */
 /*****************************************************************************/
   /* Paper size limits */
-  int		max_paper_width; /* Maximum paper width, in points */
-  int		max_paper_height; /* Maximum paper height, in points */
-  int		min_paper_width; /* Maximum paper width, in points */
-  int		min_paper_height; /* Maximum paper height, in points */
-  int		max_imageable_width; /* Maximum imageable area, in points */
-  int		max_imageable_height; /* Maximum imageable area, in points */
+  stp_dimension_t	max_paper_width; /* Maximum paper width, in points */
+  stp_dimension_t	max_paper_height; /* Maximum paper height, in points */
+  stp_dimension_t	min_paper_width; /* Maximum paper width, in points */
+  stp_dimension_t	min_paper_height; /* Maximum paper height, in points */
+  stp_dimension_t	max_imageable_width; /* Maximum imageable area, in points */
+  stp_dimension_t	max_imageable_height; /* Maximum imageable area, in points */
 /*****************************************************************************/
   /* Borders */
 				/* SHEET FED: */
 				/* Softweave: */
-  short		left_margin;	/* Left margin, points */
-  short		right_margin;	/* Right margin, points */
-  short		top_margin;	/* Absolute top margin, points */
-  short		bottom_margin;	/* Absolute bottom margin, points */
+  stp_dimension_t	left_margin;	/* Left margin, points */
+  stp_dimension_t	right_margin;	/* Right margin, points */
+  stp_dimension_t	top_margin;	/* Absolute top margin, points */
+  stp_dimension_t	bottom_margin;	/* Absolute bottom margin, points */
 				/* Printer weave: */
-  short		m_left_margin;	/* Left margin, points */
-  short		m_right_margin;	/* Right margin, points */
-  short		m_top_margin;	/* Absolute top margin, points */
-  short		m_bottom_margin;	/* Absolute bottom margin, points */
+  stp_dimension_t	m_left_margin;	/* Left margin, points */
+  stp_dimension_t	m_right_margin;	/* Right margin, points */
+  stp_dimension_t	m_top_margin;	/* Absolute top margin, points */
+  stp_dimension_t	m_bottom_margin;	/* Absolute bottom margin, points */
 				/* ROLL FEED: */
 				/* Softweave: */
-  short		roll_left_margin;	/* Left margin, points */
-  short		roll_right_margin;	/* Right margin, points */
-  short		roll_top_margin;	/* Absolute top margin, points */
-  short		roll_bottom_margin;	/* Absolute bottom margin, points */
+  stp_dimension_t	roll_left_margin;	/* Left margin, points */
+  stp_dimension_t	roll_right_margin;	/* Right margin, points */
+  stp_dimension_t	roll_top_margin;	/* Absolute top margin, points */
+  stp_dimension_t	roll_bottom_margin;	/* Absolute bottom margin, points */
 				/* Printer weave: */
-  short		m_roll_left_margin;	/* Left margin, points */
-  short		m_roll_right_margin;	/* Right margin, points */
-  short		m_roll_top_margin;	/* Absolute top margin, points */
-  short		m_roll_bottom_margin;	/* Absolute bottom margin, points */
+  stp_dimension_t	m_roll_left_margin;	/* Left margin, points */
+  stp_dimension_t	m_roll_right_margin;	/* Right margin, points */
+  stp_dimension_t	m_roll_top_margin;	/* Absolute top margin, points */
+  stp_dimension_t	m_roll_bottom_margin;	/* Absolute bottom margin, points */
 				/* Duplex margin limit (SHRT_MIN = no limit): */
-  short		duplex_left_margin;	/* Left margin, points */
-  short		duplex_right_margin;	/* Right margin, points */
-  short		duplex_top_margin;	/* Absolute top margin, points */
-  short		duplex_bottom_margin;	/* Absolute bottom margin, points */
+  stp_dimension_t	duplex_left_margin;	/* Left margin, points */
+  stp_dimension_t	duplex_right_margin;	/* Right margin, points */
+  stp_dimension_t	duplex_top_margin;	/* Absolute top margin, points */
+  stp_dimension_t	duplex_bottom_margin;	/* Absolute bottom margin, points */
 				/* Print directly to CD */
-  short		cd_x_offset;	/* Center of CD (horizontal offset) */
-  short		cd_y_offset;	/* Center of CD (vertical offset) */
-  short		cd_page_width;	/* Width of "page" when printing to CD */
-  short		cd_page_height;	/* Height of "page" when printing to CD */
+  stp_dimension_t	cd_x_offset;	/* Center of CD (horizontal offset) */
+  stp_dimension_t	cd_y_offset;	/* Center of CD (vertical offset) */
+  stp_dimension_t	cd_page_width;	/* Width of "page" when printing to CD */
+  stp_dimension_t	cd_page_height;	/* Height of "page" when printing to CD */
 				/* Extra height for form factor command */
-  short		paper_extra_bottom; /* Extra space on the bottom of the page */
+  stp_dimension_t	paper_extra_bottom; /* Extra space on the bottom of the page */
 /*****************************************************************************/
   /* Parameters for escputil */
   short		alignment_passes;
@@ -459,42 +456,40 @@ typedef struct escp2_printer
 /* From escp2-channels.c: */
 
 extern const inkname_t *stpi_escp2_get_default_black_inkset(void);
-extern int stp_escp2_load_inkgroup(const stp_vars_t *v, const char *name);
+extern int stpi_escp2_load_inkgroup(const stp_vars_t *v, const char *name);
 
 /* From escp2-papers.c: */
-extern int stp_escp2_load_media(const stp_vars_t *v, const char *name);
-extern int stp_escp2_has_media_feature(const stp_vars_t *v, const char *name);
-extern const paper_t *stp_escp2_get_default_media_type(const stp_vars_t *v);
-extern const paper_t *stp_escp2_get_media_type(const stp_vars_t *v, int ignore_res);
-extern int stp_escp2_printer_supports_rollfeed(const stp_vars_t *v);
-extern int stp_escp2_printer_supports_print_to_cd(const stp_vars_t *v);
-extern int stp_escp2_printer_supports_duplex(const stp_vars_t *v);
+extern int stpi_escp2_load_media(const stp_vars_t *v, const char *name);
+extern int stpi_escp2_has_media_feature(const stp_vars_t *v, const char *name);
+extern const paper_t *stpi_escp2_get_default_media_type(const stp_vars_t *v);
+extern const paper_t *stpi_escp2_get_media_type(const stp_vars_t *v, int ignore_res);
+extern int stpi_escp2_printer_supports_rollfeed(const stp_vars_t *v);
+extern int stpi_escp2_printer_supports_print_to_cd(const stp_vars_t *v);
+extern int stpi_escp2_printer_supports_duplex(const stp_vars_t *v);
 
-extern int stp_escp2_load_input_slots(const stp_vars_t *v, const char *name);
-extern const input_slot_t *stp_escp2_get_input_slot(const stp_vars_t *v);
+extern int stpi_escp2_load_input_slots(const stp_vars_t *v, const char *name);
+extern const input_slot_t *stpi_escp2_get_input_slot(const stp_vars_t *v);
 
-extern int stp_escp2_load_media_sizes(const stp_vars_t *v, const char *name);
-extern void stp_escp2_set_media_size(stp_vars_t *v, const stp_vars_t *src);
+extern int stpi_escp2_load_media_sizes(const stp_vars_t *v, const char *name);
+extern void stpi_escp2_set_media_size(stp_vars_t *v, const stp_vars_t *src);
 
 /* From escp2-resolutions.c: */
-extern int stp_escp2_load_resolutions(const stp_vars_t *v, const char *name);
-extern int stp_escp2_load_resolutions_from_xml(const stp_vars_t *v, stp_mxml_node_t *node);
-extern int stp_escp2_load_printer_weaves(const stp_vars_t *v, const char *name);
-extern int stp_escp2_load_printer_weaves_from_xml(const stp_vars_t *v, stp_mxml_node_t *node);
-extern int stp_escp2_load_quality_presets(const stp_vars_t *v, const char *name);
-extern int stp_escp2_load_quality_presets_from_xml(const stp_vars_t *v, stp_mxml_node_t *node);
+extern int stpi_escp2_load_resolutions(const stp_vars_t *v, const char *name,
+				       stp_mxml_node_t *node);
+extern int stpi_escp2_load_printer_weaves(const stp_vars_t *v, const char *name);
+extern int stpi_escp2_load_quality_presets(const stp_vars_t *v, const char *name);
 
 /* From print-escp2.c: */
-extern const res_t *stp_escp2_find_resolution(const stp_vars_t *v);
-extern const inklist_t *stp_escp2_inklist(const stp_vars_t *v);
+extern const res_t *stpi_escp2_find_resolution(const stp_vars_t *v);
+extern const inklist_t *stpi_escp2_inklist(const stp_vars_t *v);
 
 /* From print-escp2-data.c: */
-extern void stp_escp2_load_model(const stp_vars_t *v, int model);
-extern stpi_escp2_printer_t *stp_escp2_get_printer(const stp_vars_t *v);
-extern model_featureset_t stp_escp2_get_cap(const stp_vars_t *v,
-					    escp2_model_option_t feature);
-extern int stp_escp2_has_cap(const stp_vars_t *v, escp2_model_option_t feature,
-			     model_featureset_t class);
+extern void stpi_escp2_load_model(const stp_vars_t *v, int model);
+extern stpi_escp2_printer_t *stpi_escp2_get_printer(const stp_vars_t *v);
+extern model_featureset_t stpi_escp2_get_cap(const stp_vars_t *v,
+					     escp2_model_option_t feature);
+extern int stpi_escp2_has_cap(const stp_vars_t *v, escp2_model_option_t feature,
+			      model_featureset_t class);
 
 
 typedef struct
@@ -567,26 +562,26 @@ typedef struct
   int extra_vertical_passes;	/* Quality enhancement */
 
   /* page parameters */		/* Indexed from top left */
-  int page_left;		/* Left edge of page (points) */
-  int page_right;		/* Right edge of page (points) */
-  int page_top;			/* Top edge of page (points) */
-  int page_bottom;		/* Bottom edge of page (points) */
-  int page_width;		/* Page width (points) */
-  int page_height;		/* Page height (points) */
-  int page_true_height;		/* Physical page height (points) */
+  stp_dimension_t page_left;		/* Left edge of page (points) */
+  stp_dimension_t page_right;		/* Right edge of page (points) */
+  stp_dimension_t page_top;		/* Top edge of page (points) */
+  stp_dimension_t page_bottom;		/* Bottom edge of page (points) */
+  stp_dimension_t page_width;		/* Page width (points) */
+  stp_dimension_t page_height;		/* Page height (points) */
+  stp_dimension_t page_true_height;	/* Physical page height (points) */
+  stp_dimension_t page_true_width;	/* Physical page height (points) */
+  stp_dimension_t cd_x_offset;		/* CD X offset (micro units) */
+  stp_dimension_t cd_y_offset;		/* CD Y offset (micro units) */
+  stp_dimension_t cd_outer_radius;	/* CD radius (micro units) */
+  stp_dimension_t cd_inner_radius;	/* CD radius (micro units) */
   int page_extra_height;	/* Extra height for set_form_factor (rows) */
   int paper_extra_bottom;	/* Extra bottom for set_page_size (rows) */
-  int page_true_width;		/* Physical page height (points) */
-  int cd_x_offset;		/* CD X offset (micro units) */
-  int cd_y_offset;		/* CD Y offset (micro units) */
-  int cd_outer_radius;		/* CD radius (micro units) */
-  int cd_inner_radius;		/* CD radius (micro units) */
 
   /* Image parameters */	/* Indexed from top left */
-  int image_height;		/* Height of printed region (points) */
-  int image_width;		/* Width of printed region (points) */
-  int image_top;		/* First printed row (points) */
-  int image_left;		/* Left edge of image (points) */
+  stp_dimension_t image_height;		/* Height of printed region (points) */
+  stp_dimension_t image_width;		/* Width of printed region (points) */
+  stp_dimension_t image_top;		/* First printed row (points) */
+  stp_dimension_t image_left;		/* Left edge of image (points) */
   int image_scaled_width;	/* Width of physical printed region (dots) */
   int image_printed_width;	/* Width of printed region (dots) */
   int image_scaled_height;	/* Height of physical printed region (dots) */
@@ -623,6 +618,3 @@ extern void stpi_escp2_terminate_page(stp_vars_t *v);
 #endif
 
 #endif /* GUTENPRINT_INTERNAL_ESCP2_H */
-/*
- * End of "$Id: print-escp2.h,v 1.139 2010/12/19 02:51:37 rlk Exp $".
- */
