@@ -1,5 +1,4 @@
 /*
- * "$Id: print-weave.c,v 1.75 2009/05/24 01:53:53 rlk Exp $"
  *
  *   Softweave calculator for Gutenprint.
  *
@@ -16,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -36,25 +34,6 @@
 #include <limits.h>
 #endif
 
-#if 1
-#define ASSERTIONS
-#endif
-
-#ifdef ASSERTIONS
-#define assert(x,v)							\
-do									\
-{									\
-  if (!(x))								\
-    {									\
-      stp_eprintf(v, "Assertion %s failed! file %s, line %d.\n",	\
-		  #x, __FILE__, __LINE__);				\
-      stp_abort();							\
-    }									\
-} while (0)
-#else
-#define assert(x,v) do {} while (0)
-#endif
-
 static int
 gcd(int x, int y)
 {
@@ -453,8 +432,8 @@ make_passmap(raw_t *w, int **map, int **starts, int first_pass_number,
 	int passes_to_map = first_pass_after_map - first_pass_to_map;
 	int i;
 
-	assert(first_pass_to_map <= first_pass_after_map, w->v);
-	assert(first_pass_to_stagger <= first_pass_after_stagger, w->v);
+	STPI_ASSERT(first_pass_to_map <= first_pass_after_map, w->v);
+	STPI_ASSERT(first_pass_to_stagger <= first_pass_after_stagger, w->v);
 
 	*map = passmap = stp_malloc(passes_to_map * sizeof(int));
 	*starts = startrows = stp_malloc(passes_to_map * sizeof(int));
@@ -646,8 +625,8 @@ stpi_calculate_row_parameters(void *vw,		/* I - weave parameters */
 	int stagger = 0;
 	int extra;
 
-	assert(row >= w->first_row_printed, w->rw.v);
-	assert(row <= w->last_row_printed, w->rw.v);
+	STPI_ASSERT(row >= w->first_row_printed, w->rw.v);
+	STPI_ASSERT(row <= w->last_row_printed, w->rw.v);
 	calculate_raw_row_parameters(&w->rw,
 	                             row + w->rw.separation * w->rw.jets,
 	                             subpass, &raw_pass, &jet, &startrow);
@@ -656,11 +635,11 @@ stpi_calculate_row_parameters(void *vw,		/* I - weave parameters */
 	phantomrows = 0;
 
 	if (raw_pass < w->first_normal_pass) {
-	        assert(raw_pass >= w->first_premapped_pass, w->rw.v);
+	        STPI_ASSERT(raw_pass >= w->first_premapped_pass, w->rw.v);
 		*pass = w->pass_premap[raw_pass - w->first_premapped_pass];
 		stagger = w->stagger_premap[raw_pass - w->first_premapped_pass];
 	} else if (raw_pass >= w->first_postmapped_pass) {
-	        assert(raw_pass >= w->first_postmapped_pass, w->rw.v);
+	        STPI_ASSERT(raw_pass >= w->first_postmapped_pass, w->rw.v);
 		*pass = w->pass_postmap[raw_pass - w->first_postmapped_pass];
 		stagger = w->stagger_postmap[raw_pass
 		                             - w->first_postmapped_pass];
@@ -1492,8 +1471,8 @@ initialize_row(stp_vars_t *v, stpi_softweave_t *sw,
 }
 
 static void
-add_to_row(stp_vars_t *v, stpi_softweave_t *sw, int row, unsigned char *buf,
-	   size_t nbytes, int color, int setactive, int h_pass)
+add_to_row_1(stp_vars_t *v, stpi_softweave_t *sw, int row, unsigned char *buf,
+	     size_t nbytes, int color, int setactive, int h_pass)
 {
   const stp_linebufs_t *bufs =
     stpi_get_linebases(v, sw, sw->lineno, h_pass, sw->head_offset[color]);
@@ -1553,6 +1532,19 @@ add_to_row(stp_vars_t *v, stpi_softweave_t *sw, int row, unsigned char *buf,
     lineactive->v[color] = 1;
 }
 
+static void
+add_to_row(stp_vars_t *v, stpi_softweave_t *sw, int row, unsigned char *buf,
+	   size_t nbytes, int color, int setactive, int h_pass)
+{
+  if (!sw->head_offset)
+    {
+      stp_eprintf(v, "ERROR: %s\n", _("Fatal error!"));
+      stp_eprintf(v, "ERROR: add_to_row: sw->head_offset == NULL\n");
+      return;
+    }
+  add_to_row_1(v, sw, row, buf, nbytes, color, setactive, h_pass);
+}
+
 static void
 stpi_flush_passes(stp_vars_t *v, int flushall)
 {
@@ -1579,9 +1571,9 @@ stp_flush_all(stp_vars_t *v)
 }
 
 static void
-finalize_row(stp_vars_t *v, int row)
+finalize_row(stp_vars_t *v, stpi_softweave_t *sw)
 {
-  stpi_softweave_t *sw = get_sw(v);
+  int row = sw->lineno;
   int i,j;
   stp_dprintf(STP_DBG_ROWS, v, "Finalizing row %d...\n", row);
   for (i = 0; i < sw->oversample; i++)
@@ -1611,11 +1603,7 @@ stp_write_weave(stp_vars_t *v, unsigned char *const cols[])
 {
   stpi_softweave_t *sw = get_sw(v);
   int length = (sw->linewidth + 7) / 8;
-  stp_lineoff_t *lineoffs[STP_MAX_WEAVE];
-  stp_lineactive_t *lineactives[STP_MAX_WEAVE];
-  stp_linecount_t *linecounts[STP_MAX_WEAVE];
   stp_linebounds_t *linebounds[STP_MAX_WEAVE];
-  const stp_linebufs_t *bufs[STP_MAX_WEAVE];
   int xlength = (length + sw->horizontal_weave - 1) / sw->horizontal_weave;
   int ylength = xlength * sw->horizontal_weave;
   unsigned char *comp_ptr;
@@ -1659,16 +1647,8 @@ stp_write_weave(stp_vars_t *v, unsigned char *const cols[])
 	      if (!sw->s[i])
 		sw->s[i] = stp_zalloc(sw->bitwidth *
 				      (sw->compute_linewidth)(v, ylength));
-	      lineoffs[i] =
-		stpi_get_lineoffsets(v, sw, sw->lineno, pass, offset);
-	      linecounts[i] =
-		stpi_get_linecount(v, sw, sw->lineno, pass, offset);
-	      lineactives[i] =
-		stpi_get_lineactive(v, sw, sw->lineno, pass,offset);
 	      linebounds[i] =
 		stpi_get_linebounds(v, sw, sw->lineno, pass, offset);
-	      bufs[i] =
-		stpi_get_linebases(v, sw, sw->lineno, pass, offset);
 	    }
 
 	  if (sw->bitwidth == 2)
@@ -1705,7 +1685,7 @@ stp_write_weave(stp_vars_t *v, unsigned char *const cols[])
   sw->current_vertical_subpass++;
   if (sw->current_vertical_subpass >= sw->vertical_oversample)
     {
-      finalize_row(v, sw->lineno);
+      finalize_row(v, sw);
       sw->lineno++;
       sw->current_vertical_subpass = 0;
     }
